#######################################################################################
#######################################################################################
#
# Functions for training and addon imputation using the variant of kNN-imputation
# described in the main paper. 
#
#######################################################################################
#######################################################################################


# Given, the number 'k' of nearest neighbours, this function imputes a data set 'x' (observations in rows) 
# as described in the main paper Section 6.1 and returns also the necessary information for addon imputation.
# 'verbose' is a boolean scalar, if TRUE information on the progress of the imputation will be printed.
#
# It is a wrapper function for 'kNNImputeTrain', which performs imputation on the standardised 
# data. 

scaledkNNImputeTrain <- function (x, k, verbose = FALSE) {

  # center and scale the data matrix:
  xscale <- scale(x)
  xmeans <- matrix(nrow=nrow(x), ncol=ncol(x), data=rep(colMeans(x, na.rm=TRUE), each=nrow(x)))
  xsd <- matrix(nrow=nrow(x), ncol=ncol(x), data=rep(apply(x, 2, sd, na.rm=TRUE), each=nrow(x)))

  # apply kNN to the scaled matrix:
  imputeobjscale <- kNNImputeTrain(xscale, k=k, verbose=verbose)

  # Transform the scaled imputed values back to the original scale of the variables:
  xunscaled <- x
  xunscaled[imputeobjscale$missing.matrixTRAIN] <- (imputeobjscale$x*xsd + xmeans)[imputeobjscale$missing.matrixTRAIN]

  # Besides the scaled versions of the imputed data matrix, which is already 
  # stored in the object to return (through 'kNNImputeTrain'), also store
  # the back-transformed data matrix:
  imputeobj <- imputeobjscale
  imputeobj$Xtrainimp <- xunscaled

  # Store the means and standard deviations of the variables from the training
  # sets:
  imputeobj$meansTRAIN <- xmeans[1,]
  imputeobj$sdTRAIN <- xsd[1,]

  # Return the list with the imputed data set, and all necessary data for addon imputation:
  return(imputeobj)

}



# This function performs the addon imputation based on the output 'imputeobj' of the 
# 'scaledkNNImputeTrain'-function. 'Xtestmiss' (observations in rows) is the data matrix 
# of the test sample.
# 'verbose' is a boolean scalar, if TRUE information on the progress of the imputation 
# will be printed.
#
# It is a wrapper function for 'kNNImputeAddon' which performs the addon imputation
# on the standardised test data matrix.
# The addon imputed test data is returned.

scaledkNNImputeAddon <- function(imputeobj, Xtestmiss, verbose = FALSE) {

  # Standardise the test data using the means and standard deviations estimated on
  # the training data:
  meansTRAIN <- imputeobj$meansTRAIN
  sdTRAIN <- imputeobj$sdTRAIN

  xmeansTRAIN <- matrix(nrow=nrow(Xtestmiss), ncol=ncol(Xtestmiss), data=rep(meansTRAIN, each=nrow(Xtestmiss)))
  xsdTRAIN <- matrix(nrow=nrow(Xtestmiss), ncol=ncol(Xtestmiss), data=rep(sdTRAIN, each=nrow(Xtestmiss)))

  xscaleTEST <- (Xtestmiss - xmeansTRAIN)/xsdTRAIN

  # Perform addon imputation on the standarised test data:
  addonimputeobj <- kNNImputeAddon(imputeobj, xscaleTEST, verbose=verbose)
  Xtestaddonscale <- addonimputeobj$Xtestmiss
  missing.matrixTEST <- addonimputeobj$missing.matrixTEST

  # Retransform the addon imputed values using the means and standard deviations estimated on
  # the training data:
  Xtestaddon <- Xtestmiss
  Xtestaddon[missing.matrixTEST] <- (Xtestaddonscale*xsdTRAIN + xmeansTRAIN)[missing.matrixTEST]

  # Return the addon imputed test data matrix:
  Xtestaddon

}



# Alternated 'kNNImpute'-function from R-package 'imputation' for use in the kNNImputeAddon-function
# (called by 'scaledkNNImputeTrain').
# The missing matrix from the training data is needed for addon imputation,
# as well as the original data and the value of k. The original 'kNNImpute'-function had to
# be alternated, since this function did not return the required informations.
# The function arguments are not described here, since this function is not be called by the user.

kNNImputeTrain <- function (x, k, verbose = FALSE) 
{

    xTRAIN <- x
   
    if (k >= nrow(x)) 
        stop("k must be less than the number of rows in x")
    missing.matrixTRAIN = is.na(x)
    numMissing = sum(missing.matrixTRAIN)
    if (verbose) {
        print(paste("imputing on", numMissing, "missing values with matrix size", 
            nrow(x) * ncol(x), sep = " "))
    }
    if (numMissing == 0) {
      return(list(x = x, missing.matrixTRAIN = missing.matrixTRAIN, xTRAIN = xTRAIN, k = k))
    }
    if (verbose) 
        print("Computing distance matrix...")
    x.distTRAIN = as.matrix(dist(x, upper = TRUE))
    if (verbose) 
        print("Distance matrix complete")
    missing.rows.indices = which(apply(missing.matrixTRAIN, 1, function(i) {
        any(i)
    }))
    x.missing = (cbind(1:nrow(x), x))[missing.rows.indices, , drop=FALSE]
    x.missing.imputed = t(apply(x.missing, 1, function(i) {
        rowIndex = i[1]
        i.original = i[-1]
        if (verbose) 
            print(paste("Imputing row", rowIndex, sep = " "))
        missing.cols = which(missing.matrixTRAIN[rowIndex, ])
        if (length(missing.cols) == ncol(x)) 
            warning(paste("Row", rowIndex, "is completely missing", 
                sep = " "))
        imputed.values = sapply(missing.cols, function(j) {
            neighbor.indices = which(!missing.matrixTRAIN[, j])
            knn.ranks = order(x.distTRAIN[rowIndex, neighbor.indices])
            knn = neighbor.indices[(knn.ranks[1:k])]
            mean(x[knn, j])
        })
        i.original[missing.cols] = imputed.values
        i.original
    }))
    x[missing.rows.indices, ] = x.missing.imputed
    missing.matrixTRAIN2 = is.na(x)
    x[missing.matrixTRAIN2] = 0
    
    # Return necessary information for addon imputation:
    return(list(x = x, missing.matrixTRAIN = missing.matrixTRAIN, xTRAIN = xTRAIN, k = k))
}



# The following function takes the result of 'kNNImputeTrain' and the test data and performs addon 
# imputation on the standardised test data matrix, whereby the nearest neighbours are only searched 
# for in the training data (called by 'scaledkNNImputeAddon'). Returns the addon imputed
# standardised test data and the missing pattern of the original test data as a boolean matrix.
# The function arguments are not described here, since this function is not be called by the user.

kNNImputeAddon <- function(imputeobj, Xtestmiss, verbose = FALSE) {

  # extract objects from the result of kNNImputeTrain:
  Xtrainmiss <- imputeobj$xTRAIN
  k <- imputeobj$k
  missing.matrixTRAIN <- imputeobj$missing.matrixTRAIN

  # Calculate the pairwise distances between training and test data:
  Xwhole <- rbind(Xtrainmiss, Xtestmiss)
  distmat <- as.matrix(dist(Xwhole, upper=TRUE))

  distmattest <- distmat[(nrow(Xtrainmiss) + 1):nrow(Xwhole), 1:nrow(Xtrainmiss)]

  # Impute the test data:
  missing.matrixTEST = is.na(Xtestmiss)
  missing.rows.indicesTEST = which(apply(missing.matrixTEST, 1, function(i) {
    any(i)
  }))
  x.missingTEST = (cbind(1:nrow(Xtestmiss), Xtestmiss))[missing.rows.indicesTEST, , drop=FALSE]

  if(length(missing.rows.indicesTEST)!=0) {

    x.missing.imputedTEST = t(apply(x.missingTEST, 1, function(i) {
        rowIndex = i[1]
        i.original = i[-1]
        if (verbose) 
            print(paste("Imputing row", rowIndex, sep = " "))
         missing.cols = which(missing.matrixTEST[rowIndex, ])
        if (length(missing.cols) == ncol(missing.matrixTEST)) 
            warning(paste("Row", rowIndex, "is completely missing", 
                sep = " "))
         imputed.values = sapply(missing.cols, function(j) {
            # The neighbouring samples only from the training set are used:
            neighbor.indices = which(!missing.matrixTRAIN[, j])
            knn.ranks = order(distmattest[rowIndex, neighbor.indices])
            knn = neighbor.indices[(knn.ranks[1:k])]
            # If 'k' is too large, it might happen that are not enough
            # nearest neighbours available.
            # --> Give a warning in these cases:
            if(sum(!is.na(Xtrainmiss[knn, j])) < k) {
              warning(paste("Only", sum(!is.na(Xtrainmiss[knn, j])), "neighbours were available instead of", k))
              mean(Xtrainmiss[knn, j], na.rm=TRUE)
            }
            else {
              # The means are calculated using the training data only:
              mean(Xtrainmiss[knn, j])
            }
        })

        i.original[missing.cols] = imputed.values
        i.original
    }))

    Xtestmiss[missing.rows.indicesTEST, ] = x.missing.imputedTEST

  }
 
  return(list(Xtestmiss=Xtestmiss, missing.matrixTEST=missing.matrixTEST))

}



# The following is a wrapper function of 'scaledkNNImputeTrain' which tunes the parameter k
# in the imputation as described in the main paper. Nearest Shrunken Centroids is performed as classification method.
# The functions arguments are as follows: 'x' is the data matrix (observations in rows),
# 'y' is the binary target variable, 'k.grid' is the grid to be used in looking for the
# optimal number of nearest neighbours, 'nrep' is the number of repetitions performed in
# the CV for tuning the number of nearest neighbours, 'printprog' is a boolean scalar,
# if TRUE progress information is printed.
# Used in the case of the higher dimensional data set 'ProstatecMethyl'.

scaledkNNImputetunekHIGHDIM <- function (x, y, k.grid=1:30, nrep=2, printprog=FALSE) {

  k.errors <- matrix(nrow=nrep, ncol=length(k.grid))
  
  # grid of shrinkage parameters:
  delta.grid <- c(0.05, 0.1, 0.25, 0.5, 1, 1.5)

  # The CV is repeated 'nrep' times:
  for(nrepcount in 1:nrep) {

    # Generate random partitions into training and test sets:
    learnindmat <- GenerateLearningsets(y=y, method="CV", fold=3)@learnmatrix
    learnindlist <- as.list(as.data.frame(t(learnindmat)))
    learnindlist <- lapply(learnindlist, function(x) x[x!=0])

    # Calculate the CV error for each k value on the specified grid:
    
    for(i in seq(along=k.grid)) {
      errorstemp <- 0

      for(j in 1:length(learnindlist)) {

         # j-th iteration of the cross validation:
         
         trainind <- learnindlist[[j]]
         Xtrain <- x[trainind,]
         ytrain <- y[trainind]
         Xtest <- x[setdiff(1:nrow(x), trainind),]
         ytest <- y[setdiff(1:nrow(x), trainind)]
         imputeobjtemp <- scaledkNNImputeTrain(Xtrain, k=k.grid[i])
         Xtrainimp <- imputeobjtemp$Xtrainimp
         Xtestaddonimp <- scaledkNNImputeAddon(imputeobjtemp, Xtest)
         tunecv <- tune(Xtrainimp, ytrain, classifier = scdaCMA, grids = list(delta = delta.grid), trace=FALSE)
         delta.opt <- delta.grid[which.min(tunecv@tuneres[[1]])]
         classifier <- scdaCMApred(Xtrainimp, ytrain, delta = delta.opt, models=TRUE)

         ytesthat <- classifier(Xtestaddonimp)
         errorstemp[j] <- mean(ytest!=ytesthat)
      }
      k.errors[nrepcount, i] <- mean(errorstemp)
      if(printprog)
        cat(paste("Iteration = ", nrepcount, ", k = ", k.grid[i], sep=""), "\n")
    }

  }

  # Besides the optimised k-value, the grid and a vector with the CV errors associated with each k value
  # are returned:
  return(list(k.grid=k.grid, k.errors=k.errors, k.opt=k.grid[which.min(colMeans(k.errors, na.rm=TRUE))]))

}





# The following is a wrapper function of 'scaledkNNImputeTrain' which tunes the parameter k
# in the imputation as described in the main paper. Random Forests is performed as classification method.
# All function arguments are as in 'scaledkNNImputetunekHIGHDIM'.
# Used in case of the lower dimensional 'GenitInfCow' data sets.

scaledkNNImputetunekHIGHDIM_RF <- function (x, y, k.grid=1:30, nrep=2, printprog=FALSE) {

  k.errors <- matrix(nrow=nrep, ncol=length(k.grid))
  
  # grid of mtry values:
  mtry.grid <- ceiling(c(0.1, 0.25, 0.5, 1, 2)*sqrt(ncol(x)))

  # The CV is repeated 'nrep' times:
  for(nrepcount in 1:nrep) {

    # Generate random partitions into training and test sets:
    learnindmat <- GenerateLearningsets(y=y, method="CV", fold=3)@learnmatrix
    learnindlist <- as.list(as.data.frame(t(learnindmat)))
    learnindlist <- lapply(learnindlist, function(x) x[x!=0])

    # Calculate the CV error for each k value on the specified grid:
	
    for(i in seq(along=k.grid)) {
      errorstemp <- 0

      for(j in 1:length(learnindlist)) {
	  
         # j-th iteration of the cross validation:	  
	  
         trainind <- learnindlist[[j]]
         Xtrain <- x[trainind,]
         ytrain <- y[trainind]
         Xtest <- x[setdiff(1:nrow(x), trainind),]
         ytest <- y[setdiff(1:nrow(x), trainind)]
         imputeobjtemp <- scaledkNNImputeTrain(Xtrain, k=k.grid[i])
         Xtrainimp <- imputeobjtemp$Xtrainimp
         Xtestaddonimp <- scaledkNNImputeAddon(imputeobjtemp, Xtest)
         tunecv <- tune(Xtrainimp, ytrain, classifier = rfCMA, grids = list(mtry = mtry.grid))
         mtry.opt <- mtry.grid[which.min(tunecv@tuneres[[1]])]
         classifiercma <- rfCMA(Xtrainimp, ytrain, mtry = mtry.opt, models=TRUE)
         randfor <- classifiercma@model[[1]]

         ytesthat <- predict(object=randfor, newdata=Xtestaddonimp)
         errorstemp[j] <- mean(ytest!=ytesthat)
      }
      k.errors[nrepcount, i] <- mean(errorstemp)
      if(printprog)
        cat(paste("Iteration = ", nrepcount, ", k = ", k.grid[i], sep=""), "\n")
    }

  }

  # Besides the optimised k-value, the grid and a vector with the CV errors associated with each k value
  # are returned:  
  return(list(k.grid=k.grid, k.errors=k.errors, k.opt=k.grid[which.min(colMeans(k.errors, na.rm=TRUE))]))

}





# The CMA-function 'scdaCMA' performing Nearest Shrunken Centroids doesn't return a 'model-object', 
# so independent observations cannot be predicted.
# Because of this the function was modified to return only a function to
# predict new observations:

library(CMA)

scdaCMApred <- function (X, y, f, learnind, delta = 0.5, models = FALSE, ...) 
{
    nrx <- nrow(X)
    ly <- length(y)
    if (nrx != length(y)) 
        stop("Number of rows of 'X' must agree with length of y \n")
    if (missing(learnind)) 
        learnind <- 1:nrx
    if (length(learnind) > nrx) 
        stop("length of 'learnind' must be smaller than the number of observations. \n")
    y <- as.factor(y)
    K <- nlevels(y)
    levels(y) <- 1:K
    if (K > 2) 
        mode <- "multiclass"
    else mode <- "binary"
    y <- as.numeric(y) - 1
    Ylearn <- y[learnind]
    Xlearn <- X[learnind, ]
    centroids <- matrix(nrow = ncol(X), ncol = K)
    variances <- matrix(nrow = ncol(X), ncol = K)
    mk <- as.numeric(sqrt(1/table(Ylearn) - 1/length(learnind)))
    for (k in 1:K) {
        indk <- (Ylearn == (k - 1))
        centroids[, k] <- muk <- colMeans(Xlearn[indk, , drop = FALSE])
        variances[, k] <- colSums((Xlearn[indk, , drop = FALSE] - 
            muk)^2)
    }
    variances <- rowSums(variances)/(length(learnind) - K)
    overallmu <- colMeans(Xlearn)
    D <- ((centroids - overallmu)/mk) * variances^{
        -1/2
    }
    shrunkD <- sign(D) * pmax(abs(D) - delta, 0)
    shrunkcentroids <- overallmu + (mk * shrunkD) * variances^{
        1/2
    }
    priors <- as.numeric(-2 * log(table(Ylearn)/sum(table(Ylearn))))
    Xtest <- X[-learnind, , drop = FALSE]
    if (nrow(Xtest) == 0) {
        Xtest <- Xlearn
        y <- Ylearn
    }
    else y <- y[-learnind]
    dist <- t(apply(Xtest, 1, function(z) colSums((z - shrunkcentroids)^2/variances) + 
        priors))
    dist <- dist - rowMeans(dist)
    prob <- safeexp(-0.5 * dist)
    prob <- prob/rowSums(prob)
    yhat <- apply(prob, 1, which.max) - 1
    if (models == TRUE) 
        modd <- list(NULL)
    if (models == FALSE) 
        modd <- list(NULL)
    # new("cloutput", yhat = yhat, y = y, learnind = learnind, 
    #     prob = prob, method = "scDA", mode = mode, model = modd)

	
	# This function returned by 'scdaCMApred' can be used to classify new observations:
    function(xnew) {
      dist <- t(apply(xnew, 1, function(z) colSums((z - shrunkcentroids)^2/variances) + 
          priors))
      dist <- dist - rowMeans(dist)
      prob <- safeexp(-0.5 * dist)
      prob <- prob/rowSums(prob)
      yhat <- apply(prob, 1, which.max)
      yhat
    }
}

# To make the functions available inside 'scdaCMApred', we have to use the same environment
# as 'scdaCMA', in which these functions are contained:
environment(scdaCMApred) <- environment(scdaCMA)