#######################################################################################
#######################################################################################
#
# Functions used for normalization:
#
# 1) Functions for training and addon RMA resp. quantile normalization using the addon
#    method described in the following paper:
#
#    Kostka, D. and Spang, R. (2008). Microarray based diagnosis profits from better 
#      documentation of gene expression signatures. PLoS Computational Biology 4, e22.
# 
#    The method is implemented in the off-cran R-package 'docval'. The code from this package
#    was alternated to be compatible with our analysis. The code in this script calls
#    this alternated functions, which are sourced in the scripts in which the real data
#    analyses are conducted.
#
# 2) Functions for training and addon RMA + global VSN; Similar to 1) with the difference
#    that before the summarization step VSN with global variance stabilisation 
#    transformation is applied. When normalizing the test data the estimates of the
#    parameters a and b obtained on the training data are used.
#    "global VSN" is described in the vignette "Introduction to vsn" of the Bioconductor
#    package 'vsn' (version 3.30.0).
#
#######################################################################################
#######################################################################################


# The following function performs RMA and returns the normalized data set together 
# with the information necessary for addon normalization.
# Input parameters are the indices of the observations to be used in the training 
# sample 'trainind' and the the 'AffyBatch'-object of the whole data set 'affybatchdata'.

RMAparam <- function(trainind, affybatchdata){

   # extract the AffyBatch of the training data:
   abo <- extractAffybatch(inds=trainind,abo=affybatchdata)

   # Perform RMA:
   abg <- bg.correct(abo,'rma')
   a.nrm.rma <- normalize.AffyBatch.qnt.val(abg,'pmonly')

   # Store parameters for addon normalization:
   rmadoc <- experimentData(a.nrm.rma)@preprocessing[['val']]
   summ.rma <- summarize.val2(a.nrm.rma)
   sumdoc.rma <- experimentData(summ.rma)@preprocessing$val$probe.effects

   # Extract gene expressions:    
   exp.train.rma <- exprs(summ.rma)

   # Return the normalized training data, together with the information necessary for
   # addon normalization:
   return(list(normdata=t(exp.train.rma), rmadoc=rmadoc, sumdoc.rma=sumdoc.rma, nfeature=nrow(exp.train.rma)))

}



# The following function performs the addon normalization based on the output
# of the 'RMAparam'-function. 'testind' are the indices of the test data, 'affybatchdata' is the 'AffyBatch'-object
# of the whole data set and 'normparam' is the output of the 'RMAparam'-function.
# The test data after addon normalization is returned.

RMAaddon <- function(testind, affybatchdata, normparam) {

    rmadoc <- normparam$rmadoc
    sumdoc.rma <- normparam$sumdoc.rma
    nfeature <- normparam$nfeature

    # Test indices are renamed:
    inds <- testind

    # Perform RMA with addon quantile normalization:
    esetm.rma <- matrix(0,nrow=length(testind),ncol=nfeature)
    for(zz in 1:length(testind)){
      system.time(ab.add <- extractAffybatch(testind[zz], affybatchdata))
      abg <- bg.correct(ab.add,'rma')
      abo.nrm.rma  <- normalize.qnt.add(abg,rmadoc$mqnts)
      eset <- summarize.add2(abo.nrm.rma,sumdoc.rma)
      esetm.rma[zz,] <- t(exprs(eset))
    }
    exp.test.add.rma <- esetm.rma     

	# Return the addon normalized test data:
    exp.test.add.rma

}








# The following is a hidden function from the Bioconductor package 'vsn';
# it fits the VSN model to a matrix 'x' and returns the VSN-normalized
# matrix 'hx'; it is merely an auxiliary function used in the function
# 'RMAVSNaddon' for performing addon RMA + global VSN further down.

library("vsn")

myvsn2trsf <- function (x, p, strata = numeric(0L), hoffset = NULL, calib = "affine") 
{
    if (!is.matrix(x) || !is.numeric(x)) 
        stop("'x' must be a numeric matrix.\n")
    if (!is.array(p) || !is.numeric(p)) 
        stop("'p' must be a numeric array.\n")
    if (any(is.na(p))) 
        stop("'p' must not contain NAs.\n")
    if (length(strata) == 0L) {
        strata = rep(1L, nrow(x))
        nrstrata = 1L
    }
    else {
        if (!(is.integer(strata) && is.vector(strata) && identical(length(strata), 
            nrow(x)) && (!any(is.na(strata))))) 
            stop("'strata' must be an integer vector of length nrow(x) with no NAs.")
        nrstrata = max(strata)
    }
    if (nrstrata == 1L && length(dim(p)) == 2L) 
        dim(p) = c(1L, dim(p))
    d2 = switch(calib, affine = ncol(x), none = 1, stop(sprintf("Invalid value of 'calib': %s", 
        calib)))
    if (length(dim(p)) != 3L || dim(p)[1L] != nrstrata || dim(p)[2L] != 
        d2 || dim(p)[3L] != 2L) 
        stop("'p' has wrong dimensions.")
    hx = .Call("vsn2_trsf", x, as.vector(p), strata, calibCharToInt(calib), 
        PACKAGE = "vsn")
    dimnames(hx) = dimnames(x)
    if (!is.null(hoffset)) {
        stopifnot(length(hoffset) == nrstrata)
        hx = hx/log(2) - hoffset[strata]
    }
    return(hx)
}

environment(myvsn2trsf) <- environment(vsn2)




# The following function performs RMA + global VSN and returns the normalized data set together 
# with the information necessary for addon normalization.
# Input parameters are the indices of the observations to be used in the training 
# sample 'trainind' and the the 'AffyBatch'-object of the whole data set 'affybatchdata'.

RMAVSNparam <- function(trainind, affybatchdata){

   require("vsn")

   # extract the AffyBatch of the training data:
   abo <- extractAffybatch(inds=trainind,abo=affybatchdata)

   # Perform RMA:
   abg <- bg.correct(abo,'rma')
   a.nrm.rma <- normalize.AffyBatch.qnt.val(abg,'pmonly')

   # Store parameters for addon normalization:
   rmadoc <- experimentData(a.nrm.rma)@preprocessing[['val']]

   # With the VSN model without calibration - "global VSN":
   vsnfit <- try(vsn2(exprs(a.nrm.rma), calib="none"))
   if(class(vsnfit) != "try-error") {

     # Replace the quantile-normalized expression values in
     # 'a.nrm.rma' by the quantile-normalized expression values
     # after global VSN:
     exprs(a.nrm.rma) <- 2 ^ vsnfit@hx
	 
	 vsncoefficients <- vsnfit@coefficients
	 
   }
   else
     vsncoefficients <- NULL 
	 
   summ.rma <- summarize.val2(a.nrm.rma)
   sumdoc.rma <- experimentData(summ.rma)@preprocessing$val$probe.effects

   # Extract gene expressions:    
   exp.train.rma <- exprs(summ.rma)

   # Return the normalized training data, together with the information necessary for
   # addon normalization:
   return(list(normdata=t(exp.train.rma), rmadoc=rmadoc, sumdoc.rma=sumdoc.rma, nfeature=nrow(exp.train.rma), vsncoefficients=vsncoefficients))

}




# The following function performs the addon RMA + global VSN normalization based on the output
# of the 'RMAVSNparam'-function. 'testind' are the indices of the test data, 'affybatchdata' is the 'AffyBatch'-object
# of the whole data set and 'normparam' is the output of the 'RMAVSNparam'-function.
# The test data after addon normalization is returned.

RMAVSNaddon <- function(testind, affybatchdata, normparam) {

    rmadoc <- normparam$rmadoc
    sumdoc.rma <- normparam$sumdoc.rma
    nfeature <- normparam$nfeature
	vsncoefficients <- normparam$vsncoefficients

    # Test indices are renamed:
    inds <- testind

    # Perform RMA with addon quantile normalization:
    esetm.rma <- matrix(0,nrow=length(testind),ncol=nfeature)
    for(zz in 1:length(testind)){
      system.time(ab.add <- extractAffybatch(testind[zz], affybatchdata))
      abg <- bg.correct(ab.add,'rma')
      abo.nrm.rma  <- normalize.qnt.add(abg,rmadoc$mqnts)

	  if(!is.null(vsncoefficients)) {
	    # Perform addon global VSN to the addon quantile-normalized expression
	    # values and replace the addon quantile-normalized expression values
	    # by the addon quantile-normalized expression values after addon
	    # global VSN:
        exprs(abo.nrm.rma) <- 2 ^ myvsn2trsf(x=exprs(abo.nrm.rma), p=vsncoefficients, strata = integer(0), 
          hoffset = log2(2*scalingFactorTransformation(vsncoefficients[,,2])),
          calib = "none")
      }
 
      eset <- summarize.add2(abo.nrm.rma,sumdoc.rma)
      esetm.rma[zz,] <- t(exprs(eset))
    }
    exp.test.add.rma <- esetm.rma     

	# Return the addon normalized test data:
    exp.test.add.rma

}






# The CMA-function 'scdaCMA' performing Nearest Shrunken Centroids doesn't return a 'model-object', 
# so independent observations cannot be predicted.
# Because of this the function was modified to return only a function to
# predict new observations:

library(CMA)

scdaCMApred <- function (X, y, f, learnind, delta = 0.5, models = FALSE, ...) 
{
    nrx <- nrow(X)
    ly <- length(y)
    if (nrx != length(y)) 
        stop("Number of rows of 'X' must agree with length of y \n")
    if (missing(learnind)) 
        learnind <- 1:nrx
    if (length(learnind) > nrx) 
        stop("length of 'learnind' must be smaller than the number of observations. \n")
    y <- as.factor(y)
    K <- nlevels(y)
    levels(y) <- 1:K
    if (K > 2) 
        mode <- "multiclass"
    else mode <- "binary"
    y <- as.numeric(y) - 1
    Ylearn <- y[learnind]
    Xlearn <- X[learnind, ]
    centroids <- matrix(nrow = ncol(X), ncol = K)
    variances <- matrix(nrow = ncol(X), ncol = K)
    mk <- as.numeric(sqrt(1/table(Ylearn) - 1/length(learnind)))
    for (k in 1:K) {
        indk <- (Ylearn == (k - 1))
        centroids[, k] <- muk <- colMeans(Xlearn[indk, , drop = FALSE])
        variances[, k] <- colSums((Xlearn[indk, , drop = FALSE] - 
            muk)^2)
    }
    variances <- rowSums(variances)/(length(learnind) - K)
    overallmu <- colMeans(Xlearn)
    D <- ((centroids - overallmu)/mk) * variances^{
        -1/2
    }
    shrunkD <- sign(D) * pmax(abs(D) - delta, 0)
    shrunkcentroids <- overallmu + (mk * shrunkD) * variances^{
        1/2
    }
    priors <- as.numeric(-2 * log(table(Ylearn)/sum(table(Ylearn))))
    Xtest <- X[-learnind, , drop = FALSE]
    if (nrow(Xtest) == 0) {
        Xtest <- Xlearn
        y <- Ylearn
    }
    else y <- y[-learnind]
    dist <- t(apply(Xtest, 1, function(z) colSums((z - shrunkcentroids)^2/variances) + 
        priors))
    dist <- dist - rowMeans(dist)
    prob <- safeexp(-0.5 * dist)
    prob <- prob/rowSums(prob)
    yhat <- apply(prob, 1, which.max) - 1
    if (models == TRUE) 
        modd <- list(NULL)
    if (models == FALSE) 
        modd <- list(NULL)
    # new("cloutput", yhat = yhat, y = y, learnind = learnind, 
    #     prob = prob, method = "scDA", mode = mode, model = modd)

	
	# This function returned by 'scdaCMApred' can be used to classify new observations:
    function(xnew) {
      dist <- t(apply(xnew, 1, function(z) colSums((z - shrunkcentroids)^2/variances) + 
          priors))
      dist <- dist - rowMeans(dist)
      prob <- safeexp(-0.5 * dist)
      prob <- prob/rowSums(prob)
      yhat <- apply(prob, 1, which.max)
      yhat
    }
}

# To make the functions available inside 'scdaCMApred', we have to use the same environment
# as 'scdaCMA', in which these functions are contained:
environment(scdaCMApred) <- environment(scdaCMA)
