#######################################################################################
#######################################################################################
#
# Functions for training and addon of supervised variable selection using two-sample Wilcoxon tests.
# Required for the combination, see Appendix F (Additional file 2).
#
#######################################################################################
#######################################################################################


# When performing variable selection using the RMA-function 'GeneSelection' the 
# Wilcoxon Test function tries to compute the exact p-values for larger samples and approximate p-values 
# for smaller samples. 
# However, for reasons of comparability the same procedure should be applied to all data sets
# and for each ratio used for splitting the data into training and test sets.
# Therefore the function 'wilcoxtestcma' called by 'GeneSelection' is alternated in the RMA-package so that
# approximate p-values are always computed:

# This package is needed for changing functions inside a package:
library(R.utils)

# Original 'wilcoxtestcma'-function:
wilcoxtestcmasafe <- function (x, y) 
{
    1 - wilcox.test(x ~ y)$p.value
}

# The latter function is alternated by adding the option 'exact = FALSE' to 'wilcox.test'
# in order to always calculate approximate p-values:
mywilcoxtestcma <- function (x, y) 
{
    1 - wilcox.test(x ~ y, exact = FALSE)$p.value
}

# Replace the original function by the alternated one in the CMA-package:
reassignInPackage("wilcoxtestcma", pkgName="CMA", mywilcoxtestcma)




# This function performs variable selection using Wilcoxon Tests and returns 
# the indices of the selected variables, since these are necessary for addon variable selection.
# Input parameters are the training data matrix 'Xtrain' (observations in rows), the binary target variable
# 'ytrain' in the training data and the number 'nvar' of variables to select.

varselectwithyTrain <- function(Xtrain, ytrain, nvar = 20) {

  # extract the indices of the variables which result in the 'nvar' smallest p-values:
  varinds <- GeneSelection(Xtrain, ytrain, method = "wilcox.test", exact=FALSE)@rankings[[1]][1, 1:nvar]
  # Subset the training data using these indices:
  Xtrainsub <- Xtrain[, varinds]

  # Return the subsetted training data and the indices of the selected variables
  # for addon variable selection:
  return(list(Xtrainsub=Xtrainsub, varinds=varinds))
  
}



# This function performs addon variable selection by choosing the columns from
# the test data matrix 'Xtest' (observations in rows), which are stored as indices in the output
# 'varselectwithyobj' of the 'varselectwithyTrain'-function.

varselectwithyAddon <- function(varselectwithyobj, Xtest) {

  Xtestsub <- Xtest[, varselectwithyobj$varinds]

  Xtestsub

}
