#######################################################################################
#######################################################################################
#
# The 'compplot'-function in this script can be used to plot the first two components
# from a PCA against each other, whereby the two classes and if present different batches 
# are highlighted. This is useful for assessing the strengths of class signal or of batch
# effects.
#
#######################################################################################
#######################################################################################


# The function 'plotcomp' plots the first components of a PCA against each other, whereby
# different batches and the two classes are separated in the plot.

# This function is not to be called by the user (function arguments therefore not described), 
# but is used only within 'compplot' (below 'plotcomp'), which first performs a PCA on the 
# data matrix and then plots the first two components against each other by 'plotcomp'.
#
# The input parameters of 'compplot' are the data matrix 'data' (num. observations x
# num. variables), the batch variable 'group' as a factor variable with levels 1, 2, 3, ... and
# the target variable 'groupsy' as a factor variable with levels 1 and 2.


plotcomp <- function(xp, groups, groupsy, alpha=20, col=rainbow(length(groupsun)),
  colconv=rainbow(length(groupsun), alpha=0.25), pch=as.character(groupsun), cex=seq(0.5,1.5,length=length(groupsun)), 
  xlim, ylim, ...) {

  if(missing(xlim)) xlim <- range(xp[,1])
  if(missing(ylim)) ylim <- range(xp[,2])

  groupsun <- levels(groups)
  yun <- levels(groupsy)
  pchall <- as.character(as.numeric(groupsy==yun[2]))

  xpsub <- xp[groups==groupsun[1],]
  plot(xpsub, xlim=xlim, ylim=ylim, col=col[1], cex=0.7, pch=pchall[groups==groupsun[1]], ...)
  dens2 <- kde2d(xpsub[,1], xpsub[,2], lims=c(min(xpsub[,1])-sd(xpsub[,1]), max(xpsub[,1])+sd(xpsub[,1]), min(xpsub[,2])-
    sd(xpsub[,2]), max(xpsub[,2])+sd(xpsub[,2])))
  contour(dens2, add=TRUE, col=colconv[1], nlevels=5)

  for(i in 2:length(groupsun)) {
    xpsub <- xp[groups==groupsun[i],]
    points(xpsub[,1], xpsub[,2], col=col[i], cex=0.7, pch=pchall[groups==groupsun[i]])
    dens2 <- kde2d(xpsub[,1], xpsub[,2], lims=c(min(xpsub[,1])-sd(xpsub[,1]), max(xpsub[,1])+sd(xpsub[,1]), min(xpsub[,2])-
      sd(xpsub[,2]), max(xpsub[,2])+sd(xpsub[,2])))
    contour(dens2, add=TRUE, col=colconv[i], nlevels=5)
  }

  return(list(col=col))

}


compplot <- function(data, groups, ...) {
  require(MASS)
  data <- as.matrix(data)
  p <- ncol(data)
  n <- nrow(data)
  xpr <- prcomp(data, scale. = FALSE)
  xp <- predict(xpr)[,1:2]
  plotc <- plotcomp(xp, groups=groups, ...)
  xm <- tapply(xp[,1], groups, mean)
  ym <- tapply(xp[,2], groups, mean)
  points(xm, ym, col=plotc$col, cex = 2, pch = 18)
  abline(h=0, lty=2)
  abline(v=0, lty=2)
  cat(xm, "\n")
  cat(ym, "\n")
}