# Set a seed for reproducibility purposes:
set.seed(1210236)

# Load the data set 'ProstatecMethyl':
load("./CVIIMCodeData/DataSets_GenitInfCow_and_ProstatecMethyl/ProstatecMethyl.Rda")
# transform the target variable into a factor:
y <- as.factor(as.numeric(y))


# Source the imputation functions (found in the folder 'AddOnFunctionsetc'):
source("./CVIIMCodeData/AddOnFunctionsetc/Imputation_functions.R")

# Source the variable selection functions (found in the folder 'AddOnFunctionsetc'):
source("./CVIIMCodeData/AddOnFunctionsetc/VariableSelection_functions.R")


# Load the CMA-package:
library(CMA)

# We observed some problems with the random number generation inside 
# the repeated cross validation necessary to perform the random splittings.
# Therefore we draw random seeds beforehand to set the seed anew before each
# replication of the cross validation:
ranseed <- sample(1000000:10000000, size=100000)
count <- 1

# Number of folds and number of repetitions:
K <- 3
niter <- 300

# Grid for tuning the shrinkage parameter in Nearest Shrunken Centroids
# (not used in Combination A):
delta.grid <- c(0.05, 0.1, 0.25, 0.5, 1, 1.5)

# Grid for tuning the number of Nearest Neighbours in kNN imputation:
kgrid <- c(1:3,(1:3)*5)

# Grid for tuning mtry in Random Forests:
mtry.grid <- 1:5

# Matrices in which the fold errors from each CV repetition will be stored
#  - in case of the full and incomplete CV as well as for the step-specific 
# incomplete fold errors:
errorcomb1K3addon <- errorcomb1K3full <- 
 errorcomb1K3woimp <- errorcomb1K3wosel <- 
 errorcomb1K3wotuning <- matrix(nrow=K, ncol=niter)

 
# Perform the repeated cross validation:

for(crossvind in 1:niter) {

  # We again employ the function 'try' in combination with a 'while'-loop
  # to catch possible errors:
  anyproblems <- try(0 + "0")

  while(class(anyproblems) == "try-error") {

  # Set seed anew before each repetition:
  set.seed(ranseed[count])
  count <- count+1

  anyproblems <- try({

  # Train all steps (except the classifier itself) on the whole data set:  

  # First tune k and than impute the whole data set using the optimized value of k:
  wholeoptk <- scaledkNNImputetunekHIGHDIM(X, y, nrep=1, k.grid=kgrid)$k.opt
  Ximp <- scaledkNNImputeTrain(X, k=wholeoptk)$Xtrainimp

  # On the whole data set select the 10 variables with the smallest p-values from
  # Wilcoxon Tests:
  Xsel <- varselectwithyTrain(Ximp, y, nvar = 10)$Xtrainsub
  
  # Tune on the whole data set::
  tunecv <- tune(Xsel, y, classifier = rfCMA, grids = list(mtry = mtry.grid), nodesize = 1)
  mtryoptwhole <- mtry.grid[which.min(tunecv@tuneres[[1]])]



  # Generate a splitting for the 'crossvind'th CV:
  learnindmat <- GenerateLearningsets(y=y, method="CV", fold=K)@learnmatrix
  learnindlist <- as.list(as.data.frame(t(learnindmat)))
  learnindlist <- lapply(learnindlist, function(x) x[x!=0])

  # Initialise vectors to store the fold errors for the 'crossvind'th
  # CV for full and incomplete CV as well as for the step-specific 
  # incomplete fold errors:
  erroraddontemp <- errorfulltemp <- errorwoimptemp <- 
    errorwoseltemp <- errorwotuningtemp <- 0

  # 'crossvindth' cross validation:	
	
  for(foldind in 1:length(learnindlist)) {

     # Perform the splitting into training and test set:
	 
     trainind <- learnindlist[[foldind]]
     testind <- setdiff(1:length(y), trainind)

     Xtrain <- X[trainind,]
     ytrain <- y[trainind]

     Xtest <- X[testind,]
     ytest <- y[testind]


     # All preparation steps trained on the training set:

     # Imputation learned on the training set: First tune k, than generate
	 # an object which contains the imputed training data set as well as the information
	 # necessary to addon impute the test data:
     optktrain <- scaledkNNImputetunekHIGHDIM(Xtrain, ytrain, nrep=1, k.grid=kgrid)$k.opt
     imputeobjtrain <- scaledkNNImputeTrain(Xtrain, k=optktrain)
	 # Extract the imputed training data:
     Xtrainimpaddon <- imputeobjtrain$Xtrainimp

     # On the training set select the 10 variables with the smallest p-values from
     # Wilcoxon Tests, subset the training set accordingly and return also
	 # the incides of the selected variables:
     varselobjaddon <- varselectwithyTrain(Xtrainimpaddon, ytrain, nvar = 10)

	 # Extract the subsetted training data:
     Xtrainsubaddon <- varselobjaddon$Xtrainsub

     # Tune mtry of Random Forests on the training data:
     tunecv <- tune(Xtrainsubaddon, ytrain, classifier = rfCMA, grids = list(mtry = mtry.grid), nodesize = 1)
     mtryopt <- mtry.grid[which.min(tunecv@tuneres[[1]])]


     # Addon preprocess the test data by sequentially calling the addon functions:

     # Addon impute the test data:
     Xtestimpaddon <- scaledkNNImputeAddon(imputeobjtrain, Xtest)

     # Subset the addon imputed test data using the indices of the variables selected on the training data:
     Xtestsubaddon <- varselectwithyAddon(varselobjaddon, Xtestimpaddon)

     # Train the classifier on the training set using the mtry value optimized on the training set and
	 # predict the test samples on the test set:
     classifier <- rfCMA(rbind(Xtrainsubaddon, Xtestsubaddon), c(ytrain, ytest), learnind = 1:nrow(Xtrainsubaddon), mtry = mtryopt, 
       models=TRUE)
     ytesthat <- classifier@yhat + 1

	 
	 # Calculate the fold error for full CV:
     erroraddontemp[foldind] <- mean(ytest!=ytesthat)



     # Calculate the fold error of incomplete CV that is the one resulting when all steps 
	 # (except the classifier itself) are trained on the whole data set:  

	 # Split the data, where all steps were performed on the whole data set into training
     # and test set:	 
     Xtrainfull <- Xsel[trainind,]
     Xtestfull <- Xsel[testind,]

     # Train the classifier and predict the test samples on the test set:
     classifier <- rfCMA(rbind(Xtrainfull, Xtestfull), c(ytrain, ytest), learnind = 1:nrow(Xtrainfull), mtry = mtryoptwhole, 
       models=TRUE)
     ytesthat <- classifier@yhat + 1

	 
	 # Calculate the fold error for incomplete CV:
     errorfulltemp[foldind] <- mean(ytest!=ytesthat)

	 


	 
     # Imputation trained on the whole data set:

	 # Split the data set, which was beforehand imputed (see further above) into
	 # training and test data:
     Xtrainwoimp <- Ximp[trainind,]
     Xtestwoimp <- Ximp[testind,]

     # On the training set select the 10 variables with the smallest p-values from
     # Wilcoxon Tests, subset the training set accordingly and return also
	 # the incides of the selected variables:
     varselobjaddon <- varselectwithyTrain(Xtrainwoimp, ytrain, nvar = 10)

	 # Extract the subsetted training data:
     Xtrainsubaddonwoimp <- varselobjaddon$Xtrainsub

     # Tune mtry of Random Forests on the training data:
     tunecv <- tune(Xtrainsubaddonwoimp, ytrain, classifier = rfCMA, grids = list(mtry = mtry.grid), nodesize = 1)
     mtryopt <- mtry.grid[which.min(tunecv@tuneres[[1]])]


	 
     # Addon preprocess the test data by sequentially calling the addon functions
	 # except for that of imputation, which was done on the whole data set:
     
     # Subset the test data using the indices of the variables selected on the training data:
     Xtestsubaddonwoimp <- varselectwithyAddon(varselobjaddon, Xtestwoimp)     

     # Train the classifier on the training set using the mtry value optimized on the training set and
	 # predict the test samples on the test set:
     classifier <- rfCMA(rbind(Xtrainsubaddonwoimp, Xtestsubaddonwoimp), c(ytrain, ytest), learnind = 1:nrow(Xtrainsubaddonwoimp), mtry = mtryopt, 
       models=TRUE)
     ytesthat <- classifier@yhat + 1

     # Calculate the step-specific incomplete fold error for imputation:
     errorwoimptemp[foldind] <- mean(ytest!=ytesthat)

	 


     # Variable selection on whole data set:

     # Put training and test set back together after imputation was performed inside CV
	 # to be able to perform variable selection on the whole data set:
     Xwithimpaddon <- X
     Xwithimpaddon[trainind,] <- Xtrainimpaddon
     Xwithimpaddon[testind,] <- Xtestimpaddon
     
     # Variable selection on the whole data set:
     varselobj <- varselectwithyTrain(Xwithimpaddon, y, nvar = 10)
     Xselwhole <- varselobj$Xtrainsub
     Xtrainselwhole <- Xselwhole[trainind,]

     # Tune mtry of Random Forests on the training data:
     tunecv <- tune(Xtrainselwhole, ytrain, classifier = rfCMA, grids = list(mtry = mtry.grid), nodesize = 1)
     mtryopt <- mtry.grid[which.min(tunecv@tuneres[[1]])]


     # Addon preprocess the test data, whereby variable selection was done on the whole data set:

     # Subset the addon imputed test data using the indices of the variables selected on the
	 # whole data set ('Xtestselwhole <- Xselwhole[testind,]' would have the same effect): 
     Xtestselwhole <- varselectwithyAddon(varselobj, Xtestimpaddon)

     # Train the classifier on the training set using the mtry value optimized on the training set and
	 # predict the test samples on the test set:
     classifier <- rfCMA(rbind(Xtrainselwhole, Xtestselwhole), c(ytrain, ytest), learnind = 1:nrow(Xtrainselwhole), mtry = mtryopt, 
       models=TRUE)
     ytesthat <- classifier@yhat + 1

     # Calculate the step-specific incomplete fold error for variable selection:
     errorwoseltemp[foldind] <- mean(ytest!=ytesthat)


    
     # Tuning on whole data set:

     # Put training and test set back together after imputation and variable selection
	 # were performed inside CV to be able to perform tuning on the whole data set:
     Xwithimpseladdon <- matrix(nrow=nrow(X), ncol=10)
     Xwithimpseladdon[trainind,] <- Xtrainsubaddon
     Xwithimpseladdon[testind,] <- Xtestsubaddon
     
     # Tune mtry on whole data set:
     tunecv <- tune(Xwithimpseladdon, y, classifier = rfCMA, grids = list(mtry = mtry.grid), nodesize = 1)
     mtryoptwholewotune <- mtry.grid[which.min(tunecv@tuneres[[1]])]


    # Addon preprocess the test data, whereby tuning was done on the whole data set:

    # Train the classifier on the training set using the mtry value optimized on the whole data set and
    # predict the test samples on the test set:
    classifier <- rfCMA(rbind(Xtrainsubaddon, Xtestsubaddon), c(ytrain, ytest), learnind = 1:nrow(Xtrainsubaddon), mtry = mtryoptwholewotune, 
      models=TRUE)
    ytesthat <- classifier@yhat + 1

    # Calculate the step-specific incomplete fold error for tuning:
    errorwotuningtemp[foldind] <- mean(ytest!=ytesthat)

  }

  })
  }

  # Store the fold errors belonging to full and incomplete CV as well as the step-specific 
  # incomplete fold errors in the respective matrices:
  errorcomb1K3addon[, crossvind] <- erroraddontemp
  errorcomb1K3full[, crossvind] <- errorfulltemp
  errorcomb1K3woimp[, crossvind] <- errorwoimptemp
  errorcomb1K3wosel[, crossvind] <- errorwoseltemp
  errorcomb1K3wotuning[, crossvind] <- errorwotuningtemp

  # Print progress:
  cat(paste("Iteration", crossvind, "of", niter), "\n")

}

# Save all calculated fold errors belonging to full and incomplete CV as well as the step-specific 
# incomplete fold errors :
save(errorcomb1K3addon, errorcomb1K3full, errorcomb1K3woimp,
  errorcomb1K3wosel, errorcomb1K3wotuning, file="./CVIIMCodeData/Combinations/ResultsCombinations/comb1_1errorK3.Rda")

# Clear the workspace:
rm(list=ls())