# Set a seed for reproducibility purposes:
set.seed(47603)

# Load the data set 'ProstatecMethyl':
load("./CVIIMCodeData/DataSets_GenitInfCow_and_ProstatecMethyl/ProstatecMethyl.Rda")
y <- as.factor(as.numeric(y))

# Source the imputation functions (found in the folder 'AddOnFunctionsetc'):
source("./CVIIMCodeData/AddOnFunctionsetc/Imputation_functions.R")

# Load the CMA-package used for Nearest Shrunken Centroids classification and
# generation of data set splittings for CV:
library(CMA)

# We observed some problems with the random number generation inside 
# the repeated cross validation necessary to perform the random splittings.
# Therefore we draw random seeds beforehand to set the seed anew before each
# replication of the cross validation:
ranseed <- sample(1000000:10000000, size=100000)
count <- 1

# Number of folds and number of repetitions:
K <- 3
niter <- 300

# Grid for tuning the shrinkage parameter in Nearest Skrunken Centroids:
delta.grid <- c(0.05, 0.1, 0.25, 0.5, 1, 1.5)

# Grid for tuning the number of Nearest Neighbours in kNN imputation:
kgrid <- c(1:3,(1:3)*5)


# Matrices in which the fold errors from each CV repetition will be stored,
# the first one belongs to full CV and the second one to incomplete CV:
errorimpute1K3addon <- errorimpute1K3full <- matrix(nrow=K, ncol=niter)


# Perform the repeated cross validation:

for(crossvind in 1:niter) {

  # Very rarely it occurred that a splitting caused an error, e.g. in
  # cases where a training set in the internal cross validation used to tune k 
  # featured only observations from one class.
  # To stop the loop from breaking, when an error occurs, we employ the
  # function 'try' in combination with a 'while'-loop, by this
  # in case an error occurs inside 'try' another splitting is drawn 
  # until no error occurs anymore.
  
  anyproblems <- try(0 + "0")

  while(class(anyproblems) == "try-error") {

  # Set seed anew before each repetition:
  set.seed(ranseed[count])
  count <- count+1

  anyproblems <- try({

  # Perform imputation on the whole data set, thereby tuning also the number
  # k of nearest neighbours:
  wholeoptk <- scaledkNNImputetunekHIGHDIM(X, y, nrep=1, k.grid=kgrid)$k.opt
  Ximp <- scaledkNNImputeTrain(X, k=wholeoptk)$Xtrainimp

  # Generate a splitting for the 'crossvind'th CV:
  learnindmat <- GenerateLearningsets(y=y, method="CV", fold=K)@learnmatrix
  learnindlist <- as.list(as.data.frame(t(learnindmat)))
  learnindlist <- lapply(learnindlist, function(x) x[x!=0])

  # 'crossvind'th CV:
  
  # Initialise vectors to store the fold errors for the 'crossvind'th
  # CV for full and incomplete CV: 
  erroraddontemp <- errorfulltemp <- 0

  for(foldind in 1:length(learnindlist)) {

     # Perform the splitting into training and test set:
	
     trainind <- learnindlist[[foldind]]
     testind <- setdiff(1:length(y), trainind)

     Xtrain <- X[trainind,]
     ytrain <- y[trainind]

     Xtest <- X[testind,]
     ytest <- y[testind]


     # Addon imputation:

	 # First tune k on the training set, then call 'scaledkNNImputeTrain' to generate
	 # an object which contains the imputed training data set as well as the information
	 # necessary to addon impute the test data:
     optktrain <- scaledkNNImputetunekHIGHDIM(Xtrain, ytrain, nrep=1, k.grid=kgrid)$k.opt
     imputeobjtrain <- scaledkNNImputeTrain(Xtrain, k=optktrain)

	 # Extract the imputed training data:
     Xtrainimpaddon <- imputeobjtrain$Xtrainimp

	 # Addon impute the test data:
     Xtestimpaddon <- scaledkNNImputeAddon(imputeobjtrain, Xtest)

	 # Tune the shrinkage parameter of Nearest Shrunken Centroids and train the 
	 # classifier:
     tunecv <- tune(Xtrainimpaddon, ytrain, classifier = scdaCMA, grids = list(delta = delta.grid))
     delta.opt <- delta.grid[which.min(tunecv@tuneres[[1]])]
     classifier <- scdaCMApred(Xtrainimpaddon, ytrain, delta = delta.opt, models=TRUE)

	 # Predict the test samples on the test data after addon imputation:
     ytesthat <- classifier(Xtestimpaddon)

	 # Calculate the fold error for full CV:
     erroraddontemp[foldind] <- mean(ytest!=ytesthat)


     # Imputation trained on the whole data set:

	 # Split the data set after imputation into a training and a test set:
     Xtrainimp <- Ximp[trainind,]
     Xtestimp <- Ximp[testind,]

	 # Tune the shrinkage parameter of Nearest Shrunken Centroids and train the 
	 # classifier:
     tunecv <- tune(Xtrainimp, ytrain, classifier = scdaCMA, grids = list(delta = delta.grid))
     delta.opt <- delta.grid[which.min(tunecv@tuneres[[1]])]
     classifier <- scdaCMApred(Xtrainimp, ytrain, delta = delta.opt, models=TRUE)

	 # Predict the test samples on the test data after imputation on the whole data set:
     ytesthat <- classifier(Xtestimp)

	 # Calculate the fold error for incomplete CV:
     errorfulltemp[foldind] <- mean(ytest!=ytesthat)

  }

  })
  }

  # Store the fold errors belonging to full and incomplete CV in the respective matrices:
  errorimpute1K3addon[, crossvind] <- erroraddontemp
  errorimpute1K3full[, crossvind] <- errorfulltemp

  # Print progress:
  cat(paste("Iteration", crossvind, "of", niter), "\n")

}

# Save all calculated fold errors belonging to full and incomplete CV:
save(errorimpute1K3addon, errorimpute1K3full, file="./CVIIMCodeData/OtherStepsAndVariableSelection/ResultsOtherStepsAndVariableSelection/impute1_1errorK3.Rda")

# Clear the workspace:
rm(list=ls())

