# Set a seed for reproducibility purposes:
set.seed(1757141)

# Load the CMA-package used for classification and (incomplete) CV:
library("CMA")

# Load the data set 'ProstatecTranscr':
load("./CVIIMCodeData/DataRetrieval/DataSetsPCA/ProstatecTranscr.Rda")
#y is already there after loading

# Number of variables to keep in the filtering is equal to the largest integer less or equal to ncol(X)/2:
nvar <- floor(ncol(X)/2)

# Number of folds and number of repetitions:
K <- 3
niter <- 300

# Generate the random splittings for all CV repetitions:
cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


# For performing full CV in the presence of variable selection the function 'classification' can take a 'genesel'-object, 
# containing information on the order of the variables in the trainng sets concerning a specific (test) statistic.
# Such a 'genesel'-object can be generated with the function 'GeneSelection'. The latter, however does not support filtering by
# variance.
# The following function takes the covariate matrix 'X', the target variable 'y' and a 'learningsets'-object containing 
# the information on the partitions for the repeated CV. It generates a 'gensel'-object that is modified to contain
# information on the order of the variables concerning their variance in such a way that it can be used in 'classification'. 
# This way full CV in the presence of variable filtering by variance can be performed:

varsontrainingdata <- function(X, y, cvdatasets) {

  # Generate a 'genesel'-object with a random method, in this case "t.test":
  genesels <- GeneSelection(X, y, learningsets=cvdatasets, method = "t.test")

  # Calculate the empirical variance on each training set and for each variable:
  vars <- t(apply(cvdatasets@learnmatrix, 1, function(x) apply(X[x[x!=0],], 2, var)))
  # For each training set, give these variances ranks, starting with the highest variance to select the variable
  # with the higher variances not with the lower:
  ranks <- t(apply(vars, 1, function(x) order(-x)))
  # The variances have also to be sorted from the largest down to the lowest:
  vars <- t(apply(vars, 1, function(x) -sort(-x)))

  # 'ranks' and 'vars' are the informations necessary in 'genesels' for 'classification' to integrate the variable selection.
  # Therefore replace the corresponding original entries from 'genesels' by the modified ones:
  genesels@rankings[[1]] <- ranks
  genesels@importance[[1]] <- vars

  # Return the modified 'genesel'-object:
  genesels

}



# Full CV:

# Generate a 'genesel'-object for use in 'classification':
geneselobj <- varsontrainingdata(X, y, cvdatasets)

# Perform the full CV whereby the variables with the higher variances are kept in the training sets through 'geneselobj'.
# Diagonal Linear Discriminant Analysis is used as classifier by setting the shrinkage parameter in Nearest Shrunken
# Centroids to zero:
nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesel = geneselobj, delta = 0)
  
# Extract the fold errors and store them in a matrix:
errorvarfilterK3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)




# Incomplete CV:

# Perform the variable filtering on the whole data set: 
varinds <- order(-apply(X, 2, var))[1:nvar]
Xsub <- X[, varinds]

# Perform the CV after variable filtering (using Diagonal Linear Discriminant Analysis 
# as classification method):
nestedcrossval <- classification(Xsub, y, learningsets = cvdatasets, 
  classifier = scdaCMA, delta=0)

# Extract the fold errors and store them in a matrix: 
errorvarfilterK3full <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)


# Save all calculated fold errors belonging to full and incomplete CV:
save(errorvarfilterK3addon, errorvarfilterK3full, file="./CVIIMCodeData/OtherStepsAndVariableSelection/ResultsOtherStepsAndVariableSelection/varfiltererrorK3.Rda")

# Clear the workspace:
rm(list=ls())