# Set a seed for reproducibility purposes:
set.seed(1234)


# Determine the number of CPUs necessary:
ncpus <- max(as.numeric(sapply(list.files("./CVIIMCodeData/PrincipalComponentAnalysisStudy/Settings"),
    function(x) strsplit(strsplit(x, split="settingpca")[[1]][2], split=".Rda"))), na.rm=TRUE)

	
# Generate a seed for each CPU:
makeseed <- function()
  as.numeric(paste(c(sample(1:9, size=1), sample(0:9, size=8, replace=TRUE)), collapse=""))
seedsind <- replicate(ncpus, makeseed())


# Load 'snowfall'-Package for parallel computation:
require("snowfall")



# Function which performs the analysis on CPU number "z":

analysiscpus <- function(z) {

# Load required packages:
library("MASS")
library("CMA")

# Set seed:
set.seed(seedsind[z])

# Grid for mtry in Random Forests:
mtry.grid <- c(1, 2, 3, 5, 10)

# Load setting parameters:
eval(parse(text=paste("load(\"./CVIIMCodeData/PrincipalComponentAnalysisStudy/Settings/settingpca", z, ".Rda\")", sep="")))

# Initiate lists in which the results for the settings
# done on this CPU will be stored:
errorpcaaddonlist <- errorpcafulllist <- list()


# Loop through the settings:

for(settingind in 1:length(Kvals)) {

# Extract parameters for setting 'settingind':
dataset <- datasets[settingind]
K <- Kvals[settingind]
ncomp <- ncomps[settingind]
classifier <- classifiers[settingind]

# Load the considered data set:
eval(parse(text=paste("load(\"", dataset, "\")", sep="")))


# We observed some problems with the random number generation inside 
# the repeated cross validation necessary to perform the random splittings
# in an analysis.
# Therefore we draw random seeds beforehand to set the seed anew before each
# replication of the cross validation:
ranseed <- replicate(100000, makeseed())
count <- 1

# Number of repetitions:
niter <- 300

# Matrices in which the fold errors from each CV repetition will be stored,
# the first one belongs to full CV and the second one to incomplete CV:
errorpcaaddon <- errorpcafull <- matrix(nrow=K, ncol=niter)

# PCA on whole data set - 'prcomp' performs the singular value decomposition
# and with 'predict' we obtain the principal components, from which we only
# use the first five:
Xpcs <- predict(prcomp(X), newdata = X)[,1:ncomp, drop=FALSE]


# Perform the repeated cross validation:

for(crossvind in 1:niter) {

  # Very rarely it can occur that a splitting causes an error.
  # To stop the loop from breaking, when an error occurs, we employ the
  # function 'try' in combination with a 'while'-loop, by this
  # in case an error occurs inside 'try' another splitting is drawn 
  # until no error occurs anymore.

  anyproblems <- try(0 + "0")

  while(class(anyproblems) == "try-error") {

  # Set seed anew before each repetition:
  set.seed(ranseed[count])
  count <- count+1

  anyproblems <- try({

  # Generate a splitting for the 'crossvind'th CV:
  learnindmat <- GenerateLearningsets(y=y, method="CV", fold=K)@learnmatrix
  learnindlist <- as.list(as.data.frame(t(learnindmat)))
  learnindlist <- lapply(learnindlist, function(x) x[x!=0])

  # 'crossvind'th CV:
  
  # Initialise vectors to store the fold errors for the 'crossvind'th
  # CV for full and incomplete CV: 
  erroraddontemp <- errorfulltemp <- 0

  for(foldind in 1:length(learnindlist)) {

     # Perform the splitting into training and test set:
	
     trainind <- learnindlist[[foldind]]
     testind <- setdiff(1:length(y), trainind)

     Xtrain <- X[trainind,]
     ytrain <- y[trainind]

     Xtest <- X[testind,]
     ytest <- y[testind]


     # Full CV:

	 # Perform singular value decomposition on training data:
     pca.train <- prcomp(Xtrain)
     # Principal components for training data:
     Xtrainpcs <- predict(pca.train, newdata = Xtrain)[,1:ncomp, drop=FALSE]
	 # Predict principal components for test data:
     Xtestpcs <- predict(pca.train, newdata = Xtest)[,1:ncomp, drop=FALSE]

	 # If appropriate in the current setting: train the LDA classificator
	 # and apply it to the test data:
     if(classifier=="lda") {
	   # Train the LDA classificator:
       lda.train <- lda(Xtrainpcs, ytrain)
	   # Predict the test samples using the predicted principal components:
       ytesthat <- predict(lda.train, newdata = Xtestpcs)$class
     }

	 # If appropriate in the current setting: construct the Random Forest tuning 'mtry'
	 # and apply it to the test data:
     if(classifier=="randomforest") {
	   # Tune 'mtry:
       tunecv <- tune(Xtrainpcs, ytrain, classifier = rfCMA, grids = list(mtry = mtry.grid))
       mtry.opt <- mtry.grid[which.min(tunecv@tuneres[[1]])]

	   # Construct the Random Forest:
       classifiercma <- rfCMA(Xtrainpcs, ytrain, mtry = mtry.opt, models=TRUE)
       randfor <- classifiercma@model[[1]]

	   # Predict the test samples using the predicted principal components:
       ytesthat <- predict(object=randfor, newdata=Xtestpcs)
     }

     # Calculate the fold error for full CV:
     erroraddontemp[foldind] <- mean(ytest!=ytesthat)


     # Incomplete CV:

	 # Split the principal components obtained after singular value decomposition
     # on the whole data set into a training and a test set:
     Xtrainpcs <- Xpcs[trainind,]
     Xtestpcs <- Xpcs[testind,]

	 # If appropriate in the current setting: train the LDA classificator
	 # and apply it to the test data:
     if(classifier=="lda") {
	   # Train the LDA classificator:
       lda.train <- lda(Xtrainpcs, ytrain)
	   # Predict the test samples:   
       ytesthat <- predict(lda.train, newdata = Xtestpcs)$class
     }

	 # If appropriate in the current setting: construct the Random Forest tuning 'mtry'
	 # and apply it to the test data:	 
     if(classifier=="randomforest") {
	   # Tune 'mtry:	 
       tunecv <- tune(Xtrainpcs, ytrain, classifier = rfCMA, grids = list(mtry = mtry.grid))
       mtry.opt <- mtry.grid[which.min(tunecv@tuneres[[1]])]

	   # Construct the Random Forest:	   
       classifiercma <- rfCMA(Xtrainpcs, ytrain, mtry = mtry.opt, models=TRUE)
       randfor <- classifiercma@model[[1]]

   	   # Predict the test samples:
       ytesthat <- predict(object=randfor, newdata=Xtestpcs)
     }

     # Calculate the fold error for incomplete CV:
     errorfulltemp[foldind] <- mean(ytest!=ytesthat)

  }

  })
  }

  # Store the fold errors belonging to full and incomplete CV in the respective matrices:
  errorpcaaddon[, crossvind] <- erroraddontemp
  errorpcafull[, crossvind] <- errorfulltemp

}

# Store the results of setting 'settingind' in the
# respective lists:
errorpcaaddonlist[[settingind]] <- errorpcaaddon
errorpcafulllist[[settingind]] <- errorpcafull

}

# Because the results are stored in lists, for reasons of consistency the informations 
# on the settings are transformed into lists to:
datasetlist <- as.list(datasets)
Klist <- as.list(Kvals)
ncomplist <- as.list(ncomps)
classifierlist <- as.list(classifiers)


# Generate lists with the results and the information on the settings:
  res <- list(errorpcaaddonlist = errorpcaaddonlist,
errorpcafulllist = errorpcafulllist, 
datasetlist = datasetlist, 
Klist = Klist, 
ncomplist = ncomplist, 
classifierlist = classifierlist
)

# Save the results obtained on this CPU:
save(res, file=paste("./CVIIMCodeData/PrincipalComponentAnalysisStudy/ResultsPCA/resultspca", z, ".Rda", sep=""))

}


# Initiate cluster:
sfInit(parallel=TRUE, cpus=ncpus, type = "MPI")

# Export all objects to the CPUs:
sfExportAll()

# Perform the parallel computation:
RealdataResults <- sfLapply(1:ncpus, function(z) analysiscpus(z))

# Stop the cluster:
sfStop()

# Clear the workspace:
rm(list=ls())
