
# Load and integrate the results, where also the CVIIM_s,n,K-values are calculated:
####################################################################################

ncpus <- max(as.numeric(sapply(list.files("./CVIIMCodeData/PrincipalComponentAnalysisStudy/Settings"),
    function(x) strsplit(strsplit(x, split="settingpca")[[1]][2], split=".Rda"))), na.rm=TRUE)

errorpcaaddonlist <- errorpcafulllist <- datasetlist <- Klist <- ncomplist <- classifierlist <- list()

for(i in 1:ncpus) {

  load(paste("./CVIIMCodeData/PrincipalComponentAnalysisStudy/ResultsPCA/resultspca", i, ".Rda", sep=""))

  errorpcaaddonlist <- append(errorpcaaddonlist, res$errorpcaaddonlist)
  errorpcafulllist <- append(errorpcafulllist, res$errorpcafulllist)
  datasetlist <- append(datasetlist, res$datasetlist)
  Klist <- append(Klist, res$Klist)
  ncomplist <- append(ncomplist, res$ncomplist)
  classifierlist <- append(classifierlist, res$classifierlist)

}

datasets <- unlist(datasetlist)
Ks <- unlist(Klist)
ncomps <- unlist(ncomplist)
classifiers <- unlist(classifierlist)


# Bring the results into the right order:
orderind <- order(Ks, classifiers, ncomps, datasets)

datasets <- datasets[orderind]
Ks <- Ks[orderind]
ncomps <- ncomps[orderind]
classifiers <- classifiers[orderind]

errorpcaaddonlist <- errorpcaaddonlist[orderind]
errorpcafulllist <- errorpcafulllist[orderind]


# Function to compute the CVIIM_s,n,K-values:

# errorfull: mean error of repeated incomplete CV,
# erroraddon: mean error of repeated full CV
measure <- function(errorfull, erroraddon) {
  meas <- sapply(1 - errorfull/erroraddon, function(x) max(c(0, x)))
  meas[is.na(meas)] <- 0
  return(meas)
}

# Estimate CVIIM-values:
CVIIMvals <- mapply(function(x, y) measure(mean(x), mean(y)), errorpcafulllist, errorpcaaddonlist, SIMPLIFY = FALSE)
CVIIMvals <- unlist(CVIIMvals)

# Full and incomplete CV errors:
errorsaddon <- sapply(errorpcaaddonlist, mean)
errorsfull <- sapply(errorpcafulllist, mean)

# Generate a data frame containing the results:
results <- data.frame(K=Ks, classifier=classifiers, ncomp=ncomps, dataset=datasets, CVIIM=CVIIMvals,
  errorsaddon=errorsaddon, errorsfull=errorsfull)





# Plot: PCA study - CVIIM_s,n,K-values versus e_full,K(s)-values and 
# zero-truncated differences of e_full,K(s)- and e_incompl,K(s)-values versus e_full,K(s)-values
################################################################################################


# Each data set in 'results' will be mapped to its corresponding ID in the data description
# table in the paper, because in the plot the data sets will be identifiable by their ID:

dataidspaper <- data.frame(dataset=c("E-GEOD-37582", "ProstatecTranscr", "GSE20189", "E-GEOD-57285", 
  "E-GEOD-48153", "E-GEOD-42826", "E-GEOD-31629", "E-GEOD-33615", 
  "E-GEOD-39046", "E-GEOD-32393", "E-GEOD-42830", "E-GEOD-39345", 
  "GSE33205", "E-GEOD-36769", "E-GEOD-43329", "E-GEOD-42042", 
  "E-GEOD-25609", "GSE37356", "E-GEOD-49641", "E-GEOD-37965"), ID=13:32)
  
dataids <- data.frame(dataset=paste("./CVIIMCodeData/DataRetrieval/DataSetsPCA/", 
  dataidspaper$dataset, ".Rda", sep=""), ID=13:32)
  
  
# Zero-truncated differences:
results2 <- results
results2$diffmeasure <- ifelse(results2$errorsaddon - results2$errorsfull>0, results2$errorsaddon - results2$errorsfull, 0)

# Calculate dataset-specific means of the e_{full, K}(s)-values, the measure-values and the zero-truncated differences:
library("plyr")
resultsum2 <- ddply(results2, .variables=c("dataset"), .fun=summarise, dataseterror=mean(errorsaddon, na.rm=TRUE), datasetCVIIM=mean(CVIIM, na.rm=TRUE),
  datasetdiffmeasure=mean(diffmeasure, na.rm=TRUE))

# Map these values to the result data frame:
reorderind <- as.numeric(factor(results2$dataset, levels=resultsum2$dataset))
results2$dataseterror <- resultsum2$dataseterror[reorderind]
results2$datasetCVIIM <- resultsum2$datasetCVIIM[reorderind]
results2$datasetdiffmeasure <- resultsum2$datasetdiffmeasure[reorderind]

# Map the IDs to the corresponding data sets in the result data frame:
results2$dataID <- dataids$ID[as.numeric(factor(results2$dataset, levels=dataids$dataset))]

# For the lines connecting the individual values to their dataset-specific means we need
# to reshape the result data frame (because we use 'ggplot'):
library("reshape")
resultslong <- reshape(results2, varying=list(c("errorsaddon", "dataseterror"), c("CVIIM", "datasetCVIIM"),
  c("diffmeasure", "datasetdiffmeasure")), v.names=c("errorsaddonboth", "CVIIMboth", "diffmeasureboth"),
  direction="long")

  
# Generate the plots:
library("ggplot2")
p1 <- ggplot(data=results2, aes(x=errorsaddon, y=CVIIM)) +
  geom_text(label=as.character(as.numeric(results2$dataID)), aes(colour=dataset), size=3) + 
  geom_point(shape=1, aes(colour=dataset), size=7) + 
  geom_line(data=resultslong, aes(x=errorsaddonboth, y=CVIIMboth, group=id, color=dataset)) + 
  geom_point(data=resultsum2, aes(x=dataseterror, y=datasetCVIIM), size=3) + 
  labs(x=expression(e["full, K"](bold(s))), y=expression(CVIIM[paste(bold(s), ",n,K", sep="")])) + 
  theme(legend.position="none")
p1

p2 <- ggplot(data=results2, aes(x=errorsaddon, y=diffmeasure)) +
  geom_text(label=as.character(as.numeric(results2$dataID)), aes(colour=dataset), size=3) + 
  geom_point(shape=1, aes(colour=dataset), size=7) + 
  geom_line(data=resultslong, aes(x=errorsaddonboth, y=diffmeasureboth, group=id, color=dataset)) + 
  geom_point(data=resultsum2, aes(x=dataseterror, y=datasetdiffmeasure), size=3) + 
  labs(x=expression(e["full, K"](bold(s))), y=expression(paste("max( ", e["full, K"](bold(s)) ~~ - ~~ e["incompl, K"](bold(s)), ",   0 )", sep=""))) + 
  theme(legend.position="none")
p2

library("gridExtra")
grobframe <- arrangeGrob(p1, p2, ncol=1, nrow=2) 
grobframe

ggsave(file="./CVIIMCodeData/Plots/Figure 4.eps", plot=grobframe, width=9, height=13)









# Plot: PCA study - Dataset-specific means of the CVIIM_s,n,K- and e_full,K(s)-values 
# plotted against the sample sizes of the datasets
################################################################################################


# Map the sample sizes of the datasets to the results:
results2$datasetpaper <- sapply(results2$dataset, function(x) dataidspaper$dataset[dataids$dataset==x])
load("./CVIIMCodeData/DataRetrieval/datainfoNormPCA.Rda")
datainfopca <- datainfo[datainfo$study=="PCA",]
datainfopca$label <- factor(datainfopca$label, levels=levels(results2$datasetpaper))

results2$nsample <- sapply(results2$datasetpaper, function(x) datainfopca$nsample[datainfopca$label==x])


# Take the means over the scenarios:
resultsum2 <- ddply(results2, .variables=c("dataset", "dataID", "nsample"), .fun=summarise, dataseterror=mean(errorsaddon, na.rm=TRUE), datasetCVIIM=mean(CVIIM, na.rm=TRUE))

# Generate the plots:

p1 <- ggplot(data=resultsum2, aes(x=nsample, y=dataseterror)) +
  geom_text(label=as.character(as.numeric(resultsum2$dataID)), size=4) + stat_smooth(size = 1, se = FALSE) + geom_hline(yintercept=0, linetype=2) + 
  labs(x="n", y=expression(paste("Mean over ", e["full, K"](bold(s)), "-values", sep="")))
p1

p2 <- ggplot(data=resultsum2, aes(x=nsample, y=datasetCVIIM)) +
  geom_text(label=as.character(as.numeric(resultsum2$dataID)), size=4) + stat_smooth(size = 1, se = FALSE) + geom_hline(yintercept=0, linetype=2) + 
  labs(x="n", y=expression(paste("Mean over ", CVIIM[paste(bold(s), ",n,K", sep="")], "-values", sep="")))
p2

library("gridExtra")
grobframe <- arrangeGrob(p2, p1, ncol=2, nrow=1) 
grobframe 

ggsave(file="./CVIIMCodeData/Plots/Supplementary Figure 11.eps", plot=grobframe, width=10, height=5)








# Calculation of CVIIM_s,n,K-values and global CVIIM-estimates
# for the respective table in the main paper
###############################################################

settingvar <- paste(results$K, results$classifier, results$ncomp, sep="_")

CVIIMmeans2 <- sapply(unique(settingvar), function(x) measure(mean(results$errorsfull[settingvar==x]), mean(results$errorsaddon[settingvar==x])))

CVIIMmeans2mat <- matrix(nrow=3, ncol=8, data=CVIIMmeans2, byrow=TRUE)
rownames(CVIIMmeans2mat) <- paste("K = ", c(3,5,10), sep="")
colnames(CVIIMmeans2mat) <- paste(rep(c("LDA", "RF"), each=4), ", ncomp = ", rep(c(2,5,10,15), times=2), sep="")

CVIIMmeansmat <- t(CVIIMmeans2mat)


globalcviimvals <- as.vector(CVIIMmeansmat)

smallglobalcviim <- (globalcviimvals <= 0.00005) & (globalcviimvals != 0)

globalcviims <- format(round(globalcviimvals, 4), nsmall=4)
globalcviims[smallglobalcviim] <- "< 0.0001"

CVIIMmeansmat2 <- matrix(nrow=nrow(CVIIMmeansmat), ncol=ncol(CVIIMmeansmat), data=globalcviims)
rownames(CVIIMmeansmat2) <- rownames(CVIIMmeansmat)
colnames(CVIIMmeansmat2) <- colnames(CVIIMmeansmat)

CVIIMmeansmat2


# LaTeX-table:

tablelatex <- c("\\begin{table}",
"\\begin{tabular}{l | l | r | r | r }",
"Classification & number of & $K = 3$ & $K = 5$ & $K = 10$\\\\",
"method & components & & & \\\\",
"\\hline",
paste("LDA & ncomp = 2 & ", paste(CVIIMmeansmat2[1,], collapse=" & "), "\\\\", sep=""), 
paste("& ncomp = 5 & ", paste(CVIIMmeansmat2[2,], collapse=" & "), "\\\\", sep=""), 
paste("& ncomp = 10 & ", paste(CVIIMmeansmat2[3,], collapse=" & "), "\\\\", sep=""), 
paste("& ncomp = 15 & ", paste(CVIIMmeansmat2[4,], collapse=" & "), "\\\\", sep=""), 
"\\hline",
paste("RF & ncomp = 2 & ", paste(CVIIMmeansmat2[5,], collapse=" & "), "\\\\", sep=""), 
paste("& ncomp = 5 & ", paste(CVIIMmeansmat2[6,], collapse=" & "), "\\\\", sep=""), 
paste("& ncomp = 10 & ", paste(CVIIMmeansmat2[7,], collapse=" & "), "\\\\", sep=""), 
paste("& ncomp = 15 & ", paste(CVIIMmeansmat2[8,], collapse=" & "), "\\\\", sep=""), 
"\\end{tabular}",
"\\caption{Estimates of global CVIIM from PCA study}", 
"\\label{t:pcaglobalcviim}",
"\\end{table}")

for(i in seq(along=tablelatex))
  cat(tablelatex[i], "\n")








# Plot: CVIIM_s,n,K-values from PCA study
##########################################

# The green diamonds are depicted with 'geom_polygon' for which reason we have
# to generate a data.frame describing the corners of the polygons:

sizex <- 0.15; sizey <- 0.02

nplots <- length(unique(results$ncomp))*length(unique(results$classifier))

xdiamonds <- rep(as.vector(sapply(1:3, function(x) x + c(-sizex, 0, sizex, 0))), times=nplots)

globalcviims <- as.vector(CVIIMmeans2mat)

ydiamonds <- as.vector(sapply(1:length(globalcviims), function(x) globalcviims[x] + c(0, sizey, 0, -sizey)))

Kseq <- c(3,5,10)
Krep <- rep(rep(Kseq, each=4), times=nplots)

classifiercol2 <- sapply(colnames(CVIIMmeans2mat), function(x) strsplit(x, split=",")[[1]][1])
classifierrep <- rep(rep(classifiercol2, each=nrow(CVIIMmeans2mat)), each=4)

ncompcol <- sapply(colnames(CVIIMmeans2mat), function(x) strsplit(x, split=", ")[[1]][2])
ncomprep <- rep(rep(ncompcol, each=nrow(CVIIMmeans2mat)), each=4)

datatemp <- data.frame(x=xdiamonds, y=ydiamonds, K=Krep, classifier=classifierrep, ncomp=ncomprep)


# For optical reasons we change the labels of 'classifier' and 'ncomp':

results2 <- results
results2$classifier <- as.character(results2$classifier)
results2$classifier[results2$classifier=="lda"] <- "LDA"
results2$classifier[results2$classifier=="randomforest"] <- "RF"

results2$ncomp <- paste("ncomp = ", results2$ncomp, sep="")
results2$ncomp <- factor(results2$ncomp, levels=c("ncomp = 2", "ncomp = 5", "ncomp = 10", "ncomp = 15"))


datatemp2 <- datatemp
datatemp2$classifier <- as.character(datatemp2$classifier)
datatemp2$classifier[datatemp2$classifier=="lda"] <- "LDA"
datatemp2$classifier[datatemp2$classifier=="randomforest"] <- "RF"

datatemp2$ncomp <- factor(datatemp2$ncomp, levels=c("ncomp = 2", "ncomp = 5", "ncomp = 10", "ncomp = 15"))


# Generate the plot:

p <- ggplot(data=results2, aes(x=as.factor(as.numeric(as.factor(K))), y=CVIIM)) +
  geom_boxplot() + geom_line(aes(x=as.numeric(as.factor(K)), y=CVIIM, group=dataset), color="grey") + geom_boxplot(fill="transparent") + facet_wrap(~ classifier + ncomp, nrow=4, ncol=2) + scale_x_discrete(breaks=1:3, labels=paste("K = ", Kseq, sep="")) + 
  labs(x="", y=expression(CVIIM[paste(bold(s), ",n,K", sep="")]))  + 
  geom_polygon(data=datatemp2, aes(x,y, group=K), fill="green3")

p

ggsave(file="./CVIIMCodeData/Plots/Figure 3.eps", width=7, height=10)








# Plot: e_full,K(s)- and e_incompl,K(s)-values for all datasets and settings from the PCA study
###############################################################################################

# Generate a data.frame tailored to the plot problem:

errdist <- 0.6

resultslong2 <- reshape(results2, varying=list(c("errorsaddon", "errorsfull")), 
  v.names="errors", direction="long")
resultslong2$x <- as.numeric(as.factor(resultslong2$K))
resultslong2$x[resultslong2$time==1] <- resultslong2$x[resultslong2$time==1] - errdist/2
resultslong2$x[resultslong2$time==2] <- resultslong2$x[resultslong2$time==2] + errdist/2


# Generate the plot:

p <- ggplot(data=resultslong2, aes(x=factor(K), y=errors)) +
  geom_line(aes(x=x, group=id)) + facet_wrap(~ classifier + ncomp, nrow=4, ncol=2) +
  geom_point(aes(x=x, shape=factor(ifelse(time==1, 4, 1)))) + 
  labs(x="", y="") + scale_shape_discrete(name="", breaks=c(1, 4), labels=c(expression(e["incompl, K"](bold(s))), expression(e["full, K"](bold(s))))) + 
  scale_x_discrete(breaks=1:3, labels=paste("K = ", Kseq, sep=""), expand=c(0.05,-2)) + 
  theme(legend.position="left", legend.key.height=unit(3,"line")) +
  guides(shape=guide_legend(label.position="bottom", label.vjust = 0.5, label.hjust = 0.5, label.theme = element_text(angle = 90)))

p

ggsave(file="./CVIIMCodeData/Plots/Figure 5.eps", width=7, height=10)











# Plot: Boxplots of the e_{full, K}(s)- and e_{incompl, K}(s)-values separate after value 
# of K for the settings with Linear Discriminant Analysis as classifier. The lines connect the 
# values corresponding to one data set. The red lines in each case mark those three results which were 
# associated with the largest differences between e_{full, 3}(s) and e_{full, 10}(s):
###################################################################################################

settingvar <- paste(results$classifier, results$ncomp, sep="_")

dev.new(width=7, height=15)


mains <- paste("LDA, ncomp =", rep(c(2, 5, 10, 15), each=2))
mains <- paste("expression(paste(\"", mains, " - \",", rep(c(" e[\"full, K\"](bold(s))", " e[\"incompl, K\"](bold(s))"), times=4), ", sep=\"\"))", sep="")


par(mfrow=c(4,2))
for(i in 1:4) {

resultssub <- results[settingvar == unique(settingvar)[i],]

eval(parse(text=paste("boxplot(resultssub$errorsaddon ~ resultssub$K, xlab=\"K\", main=", mains[i*2 - 1], ")", sep="")))
diffs <- sapply(unique(resultssub$dataset), function(x) resultssub$errorsaddon[resultssub$dataset==x][resultssub$K[resultssub$dataset==x]==3] -
  resultssub$errorsaddon[resultssub$dataset==x][resultssub$K[resultssub$dataset==x]==10])
baddatasets <- unique(resultssub$dataset)[order(diffs)[(length(diffs)-2):length(diffs)]]
for(j in 1:20) {
  if(unique(resultssub$dataset)[j] %in% baddatasets)
    lines(1:3, resultssub$errorsaddon[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col=2)
  else
    lines(1:3, resultssub$errorsaddon[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col="grey")
}
boxplot(resultssub$errorsaddon ~ resultssub$K, add=TRUE)

eval(parse(text=paste("boxplot(resultssub$errorsfull ~ resultssub$K, xlab=\"K\", main=", mains[i*2], ")", sep="")))
for(j in 1:20) {
  if(unique(resultssub$dataset)[j] %in% baddatasets)
    lines(1:3, resultssub$errorsfull[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col=2)
  else
    lines(1:3, resultssub$errorsfull[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col="grey")
}
boxplot(resultssub$errorsfull ~ resultssub$K, add=TRUE)


}
par(mfrow=c(1,1))  




# Plot: Boxplots of the e_{full, K}(s)- and e_{incompl, K}(s)-values separate after value of K for the settings with 
# Random Forests as classifier. The lines connect the values corresponding
# to one data set. The red lines in each case mark those three results which were 
# associated with the largest differences between e_{full, 3}(s) and e_{full, 10}(s):
###########################################################################################################

settingvar <- paste(results$classifier, results$ncomp, sep="_")

dev.new(width=7, height=15)


mains <- paste("RF, ncomp =", rep(c(2, 5, 10, 15), each=2))
mains <- paste("expression(paste(\"", mains, " - \",", rep(c(" e[\"full, K\"](bold(s))", " e[\"incompl, K\"](bold(s))"), times=4), ", sep=\"\"))", sep="")


par(mfrow=c(4,2))
for(i in 5:8) {

resultssub <- results[settingvar == unique(settingvar)[i],]

eval(parse(text=paste("boxplot(resultssub$errorsaddon ~ resultssub$K, xlab=\"K\", main=", mains[(i-4)*2 - 1], ")", sep="")))
diffs <- sapply(unique(resultssub$dataset), function(x) resultssub$errorsaddon[resultssub$dataset==x][resultssub$K[resultssub$dataset==x]==3] -
  resultssub$errorsaddon[resultssub$dataset==x][resultssub$K[resultssub$dataset==x]==10])
baddatasets <- unique(resultssub$dataset)[order(diffs)[(length(diffs)-2):length(diffs)]]
for(j in 1:20) {
  if(unique(resultssub$dataset)[j] %in% baddatasets)
    lines(1:3, resultssub$errorsaddon[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col=2)
  else
    lines(1:3, resultssub$errorsaddon[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col="grey")
}
boxplot(resultssub$errorsaddon ~ resultssub$K, add=TRUE)

eval(parse(text=paste("boxplot(resultssub$errorsfull ~ resultssub$K, xlab=\"K\", main=", mains[(i-4)*2], ")", sep="")))
for(j in 1:20) {
  if(unique(resultssub$dataset)[j] %in% baddatasets)
    lines(1:3, resultssub$errorsfull[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col=2)
  else
    lines(1:3, resultssub$errorsfull[resultssub$dataset==unique(resultssub$dataset)[j]][order(resultssub$K[resultssub$dataset==unique(resultssub$dataset)[j]])], col="grey")
}
boxplot(resultssub$errorsfull ~ resultssub$K, add=TRUE)


}
par(mfrow=c(1,1))  
  



  


# As described in the main paper we performed a sensitivity analysis, 
# in each setting leaving out the one and the three data set(s) with the largest differences
# between e_{full, 3}(s) and e_{full, 10}(s) and then recalculating the 
# global CVIIM-estimate:
############################################################################################

settresults <- list()


for(sett in 1:length(unique(settingvar))) {

settresults[[sett]] <- matrix(nrow=3, ncol=3)


resultssub <- results[settingvar == unique(settingvar)[sett],]
settresults[[sett]][1,] <- sapply(unique(resultssub$K), function(x) measure(mean(resultssub$errorsfull[resultssub$K==x]), mean(resultssub$errorsaddon[resultssub$K==x])))


diffs <- sapply(unique(resultssub$dataset), function(x) resultssub$errorsaddon[resultssub$dataset==x][resultssub$K[resultssub$dataset==x]==3] -
  resultssub$errorsaddon[resultssub$dataset==x][resultssub$K[resultssub$dataset==x]==10])


baddatasets <- unique(resultssub$dataset)[order(diffs)[length(diffs)]]

resultssub2 <- resultssub[!(resultssub$dataset %in% baddatasets),]
settresults[[sett]][2,] <- sapply(unique(resultssub2$K), function(x) measure(mean(resultssub2$errorsfull[resultssub2$K==x]), mean(resultssub2$errorsaddon[resultssub2$K==x])))



baddatasets <- unique(resultssub$dataset)[order(diffs)[(length(diffs)-2):length(diffs)]]

resultssub2 <- resultssub[!(resultssub$dataset %in% baddatasets),]
settresults[[sett]][3,] <- sapply(unique(resultssub2$K), function(x) measure(mean(resultssub2$errorsfull[resultssub2$K==x]), mean(resultssub2$errorsaddon[resultssub2$K==x])))

}

names(settresults) <- unique(settingvar)


settresults <- lapply(settresults, round, digits=4)
settresults <- lapply(settresults, function(x) {rownames(x) <- c("All 20 data sets", "Leaving one out", "Leaving three out");
  colnames(x) <- c("K = 3", "K = 5", "K = 10"); x})
names(settresults) <- paste(rep(c("LDA,", "RF,"), each=4), "ncomp =", rep(c(2, 5, 10, 15), times=2))

settresults
