###############################################################################
#
#   This R script contains a demonstrative analysis where CVIIM for selecting
#   10 variables in a supervised fashion is estimated and depicted graphically
#   for a simple simulated dataset.
#   The purpose of this code is exclusively to provide a simple illustration
#   of how a possible implementation of an analysis using CVIIM may look like.
#   We implemented our analyses using the same structure as the one presented
#   here. However, the analyses on real data are more complicated, spread over
#   different R scripts and most often involve parallelization for which reason 
#   the corresponding code is less easy to follow than that of the artificial 
#   example presented here.
#
#   Note: The resulting CVIIM-estimates of this demonstrative analysis are not 
#   intended to be insightful in any form.
#
###############################################################################



# Simulate dataset:
###################

# Set seed for reproducibility:
set.seed(1234)


# Target variable with 15 observations from either class:
y <- factor(sample(rep(1:2, times=c(15,15))))
y

# Covariate matrix with 100 covariates where 15 are informative:
X <- matrix(nrow=30, ncol=100, data=rnorm(30*100))
X[y==2,1:15] <- X[y==2,1:15] + 0.2





# Functions for variable selection and addon variable selection:
################################################################


# Function for performing variable selection on the training dataset

# Input:   Xtrain   Covariate matrix in the training dataset (observations in rows)
#          ytrain   vector of target variable in the training dataset, where
#                   the classes have to be denoted as '1' and '2'

VariableSelectionParam <- function(Xtrain, ytrain) {

  # For each variable perform a t-test between the classes and return the p-value:
  pvals <- apply(Xtrain, 2, function(x, y) t.test(x[y==1], x[y==2])$p.value, y=ytrain)
  # Indices of the 10 variables with the smallest p-values:
  selind <- order(pvals)[1:10]

  # Subset the training data to feature only the 10 variables which
  # where connected with the smallest p-values from the t-tests:
  Xtrainsub <- Xtrain[,selind]

  # Return: 1) The subsetted (i.e. prepared) training dataset;
  # 2) all information necessary for addon-variable selection 'params',
  # i.e. the indices of the selected variables 'selind':
  return(list(X=Xtrainsub, params=list(selind=selind)))

}


# Function for addon-variable selection on the test dataset

# Input:   Xtest    Covariate matrix in the test dataset (observations in rows)
#          params   Object 'params' returned by the function 'VariableSelectionParam';
#                   contains all information necessary for addon-variable selection,
#                   i.e. the indices of the selected variables 'selind'

VariableSelectionAddon <- function(Xtest, params) {

  # Subset the test dataset using the indices of the variables
  # determined on the training dataset:
  Xsubtest <- Xtest[,params$selind]

  # Return the subsetted (i.e. prepared) test dataset:
  return(Xsubtest)

}




# Full CV and Incomplete CV:
############################


# Package 'CMA'; needed here for generating the splittings for
# the repeated CV and for performing Linear Discriminant Analysis:
library("CMA")

# Number of repetitions of the full and incomplete CV: 
B <- 20

# Vector of considered values for 'K', the number of folds in CV:
Kseq <- c(3,5)


# Perform full and incomplete CV - at first for K = 3:

# Set K = 3:
K <- Kseq[1]

# Matrices where the fold errors from full and incomplete CV will
# be stored:
fullcverrormatK3 <- incomplcverrormatK3 <- matrix(nrow=K, ncol=B)
# NOTE: In all analyses on real data we used the substring 'addon'
# for full CV and the substring 'full' for incomplete CV. 


# Perform variable selection beforehand on the whole dataset:
VarSeltrain <- VariableSelectionParam(Xtrain=X, ytrain=y)
Xsub <- VarSeltrain$X


# Full and incomplete CV:

for(crossvind in 1:B) {

  # Generate splitting for the 'crossvind'th CV:
  learnindmat <- GenerateLearningsets(y=y, method="CV", fold=K)@learnmatrix
  learnindlist <- as.list(as.data.frame(t(learnindmat)))
  learnindlist <- lapply(learnindlist, function(x) x[x!=0])

  
  # 'crossvindth' full and incomplete CV:

  fullcverrors <- incomplcverrors <- 0

  for(foldind in 1:length(learnindlist)) {

    # Split into training and test sets:
	
    trainind <- learnindlist[[foldind]]
    testind <- setdiff(1:length(y), trainind)

    Xtrain <- X[trainind,]
    ytrain <- y[trainind]

    Xtest <- X[testind,]
    ytest <- y[testind]


	# Variable selection with addon (full CV):
	
	# Perform variable selection on training dataset:
    VarSeltrain <- VariableSelectionParam(Xtrain=Xtrain, ytrain=ytrain)

    # Subset training dataset:
    Xtrainsubfull <- VarSeltrain$X

	# Subset test dataset by the variables selected on the training 
	# data (addon-variable selection):
    Xtestsubfull <- VariableSelectionAddon(Xtest, params=VarSeltrain$params)

    # Calculate test set error of LDA:
    ytesthat <- ldaCMA(X=rbind(Xtrainsubfull, Xtestsubfull), y=c(ytrain, ytest), learnind=1:length(trainind))@yhat + 1

	# Calculate fold error of full CV:
    fullcverrors[foldind] <- mean(ytest!=ytesthat)


	# Variable selection on the whole dataset (incomplete CV):

	# Subset training dataset where variable selection had been performed on
	# whole dataset:
    Xtrainsubincompl <- Xsub[trainind,]

	# Subset test dataset where variable selection had been performed on
	# whole dataset:
    Xtestsubincompl <- Xsub[testind,]

    # Calculate (biased) test set error of LDA:
    ytesthat <- ldaCMA(X=rbind(Xtrainsubincompl, Xtestsubincompl), y=c(ytrain, ytest), learnind=1:length(trainind))@yhat + 1

	# Calculate fold error of incomplete CV:
    incomplcverrors[foldind] <- mean(ytest!=ytesthat)

  }

  # Store the results of the 'crossvindth' full and incomplete CV
  # in the respective matrices:
  fullcverrormatK3[, crossvind] <- fullcverrors
  incomplcverrormatK3[, crossvind] <- incomplcverrors

  # Print progress:
  cat(paste("Iteration", crossvind, "of", B), "\n")

}



# Perform full and incomplete CV for K = 5 (uncommented because analoguous to case
# with K = 3):

K <- Kseq[2]

fullcverrormatK5 <- incomplcverrormatK5 <- matrix(nrow=K, ncol=B)

VarSeltrain <- VariableSelectionParam(Xtrain=X, ytrain=y)
Xsub <- VarSeltrain$X

for(crossvind in 1:B) {

  learnindmat <- GenerateLearningsets(y=y, method="CV", fold=K)@learnmatrix
  learnindlist <- as.list(as.data.frame(t(learnindmat)))
  learnindlist <- lapply(learnindlist, function(x) x[x!=0])


  fullcverrors <- incomplcverrors <- 0

  for(foldind in 1:length(learnindlist)) {

    trainind <- learnindlist[[foldind]]
    testind <- setdiff(1:length(y), trainind)

    Xtrain <- X[trainind,]
    ytrain <- y[trainind]

    Xtest <- X[testind,]
    ytest <- y[testind]


    VarSeltrain <- VariableSelectionParam(Xtrain=Xtrain, ytrain=ytrain)
    Xtrainsubfull <- VarSeltrain$X

    Xtestsubfull <- VariableSelectionAddon(Xtest, params=VarSeltrain$params)


    ytesthat <- ldaCMA(X=rbind(Xtrainsubfull, Xtestsubfull), y=c(ytrain, ytest), learnind=1:length(trainind))@yhat + 1

    fullcverrors[foldind] <- mean(ytest!=ytesthat)



    Xtrainsubincompl <- Xsub[trainind,]

    Xtestsubincompl <- Xsub[testind,]


    ytesthat <- ldaCMA(X=rbind(Xtrainsubincompl, Xtestsubincompl), y=c(ytrain, ytest), learnind=1:length(trainind))@yhat + 1

    incomplcverrors[foldind] <- mean(ytest!=ytesthat)

  }

  fullcverrormatK5[, crossvind] <- fullcverrors
  incomplcverrormatK5[, crossvind] <- incomplcverrors

  cat(paste("Iteration", crossvind, "of", B), "\n")

}





# CVIIM-related functions:
###########################


# Function to compute the CVIIM_s,n,K,b-values:

# Input:   incomplcverror   errors of incomplete CV
#          fullcverror      errors of full CV

measureb <- function(incomplcverror, fullcverror) {
  meas <- 1 - incomplcverror/fullcverror
  return(meas)
}


# Function to compute the CVIIM_s,n,K-values:

# Input:   incomplcverror   mean error of repeated incomplete CV
#          fullcverror      mean error of repeated full CV

measure <- function(incomplcverror, fullcverror) {
  meas <- sapply(1 - incomplcverror/fullcverror, function(x) max(c(0, x)))
  meas[is.na(meas)] <- 0
  return(meas)
}


# Function to compute CVIIM_s,n,K together with
# 25%- and 75%-quartile of the CVIIM_s,n,K,b-values:

# Input:   incomplcverror   K x B matrix of fold errors of repeated incomplete CV
#          fullcverror      K x B matrix of fold errors of repeated full CV

measurewithquart <- function(incomplcverror, fullcverror) {

  # Compute the CVIIM_s,n,K,b-values:
  measurei <- mapply(function(x, y) measureb(mean(x), mean(y)), data.frame(incomplcverror), data.frame(fullcverror))
  # Compute 25%- and 75%-quartile of the CVIIM_s,n,K,b-values:
  quartiles <- quantile(measurei, c(0.25, 0.75))
  # Compute the CVIIM_s,n,K-values:
  measureval <- measure(mean(incomplcverror), mean(fullcverror))
  # Return CVIIM_s,n,K-values and 25%- and 75%-quartile of the CVIIM_s,n,K,b-values:
  return(list(cviim=measureval, quartiles=quartiles))

}




# Calculate the CVIIM-estimates and derive further necessary information:
##########################################################################


# Compute CVIIM_s,n,K together with 25%- and 75%-quartile of the 
# CVIIM_s,n,K,b-values for K = 3 and K = 5; these values will
# be depicted in the plot at the end of the script

cviimwithquartiles <- list(measurewithquart(incomplcverrormatK3, fullcverrormatK3),
  measurewithquart(incomplcverrormatK5, fullcverrormatK5))

cviimwithquartiles



# Store CVIIM-estimates with quartiles in a data frame together
# with meta-informations such as used classifier, used dataset
# and further informations on the setting:

# Vector of dataset names:
datasets <- "SimulData"

results <- data.frame(method=factor(rep("LDA", each=2)),
dataset=factor(rep(datasets, each=2)),
K=c(3,5),
CVIIM=sapply(cviimwithquartiles, function(x) x$cviim),
CVIIM25=sapply(cviimwithquartiles, function(x) x$quartiles[1]),
CVIIM75=sapply(cviimwithquartiles, function(x) x$quartiles[2]))

# Numbers to use for datasets in plot:
results$dataID <- c(1,1)

results





# Generate an error bar plot of the CVIIM_s,n,K-values together with 25%- and 75%-quartiles 
# of the CVIIM_s,n,K,b-values:
#############################################################################################

# Use 'ggplot2'-package to obtain a pleasant and well-structured result:
library("ggplot2")

# Horizontal space between numbers representing datasets and corresponding error bars:
spacingnumbers1 <- 0.05

# horizontal position of numbers representing datasets:
results$x <- as.numeric(as.factor(results$K))

# Generate the plot:
p <- ggplot(data=results, aes(x=factor(K), y=CVIIM)) + 
  # Add CVIIM-estimates:
  geom_text(aes(x=x,label=as.character(results$dataID), colour=dataset), size=6) +
  # Add lines connecting values corresponding to the same dataset:
  geom_line(aes(x=x, y=CVIIM, color=dataset)) +
  # Suppress legend:
  theme(legend.position="none") + 
  # Add error bars:
  geom_errorbar(aes(x=x+spacingnumbers1, ymin=CVIIM25, ymax=CVIIM75, colour=dataset), width=0.15) +
  # Add horizontal dashed line at the origin (corresponds to no bias by incomplete CV):
  geom_hline(yintercept=0, linetype=2) + 
  # Add axis labels:
  labs(x="", y=expression(CVIIM[paste(bold(s), ",n,K", sep="")])) + 
  # Add axis tick labels:
  scale_x_discrete(breaks=1:2, labels=paste("K =", c(3,5)), expand=c(0.55,-1))
  
# Depict the plot: 
p

# Save the plot:
# ggsave(file="./CVIIMCodeData/Plots/CVIIMRuntimeExample.eps", width=6, height=4)
