##########################################################################
#                                                                        #
#   In this file we set up the two scenarios 'strong signal' and         #
#   'weak signal' (see variable 'MeanCov' in Additional file 2).         #
#   These are stored in the files "simucorbigsignal.RData" and           #
#   "simucorweaksignal.RData" found in the folder                        #
#   "SimulationCVIIM/confs".                                             #
#                                                                        # 
##########################################################################



# Scenario with block-diagonal covariance matrix and strong signal:
###################################################################

# set seed for reproducibility:
set.seed(8122)

# number of variables to simulate:
p <- 2000

# Load the data set:
load("./CVIIMCodeData/DataRetrieval/DataSetsPCA/ProstatecTranscr.Rda")

# Proportion of variables with signal:
signal <- 0.1

# Number of variables with signal:
p1 <- ceiling(p*signal)

# Number of variables without signal:
p2 <- p - p1


# We preselect the 2000 variables with the highest t-statistics
# from two-sample-tests between the two groups:

yp <- as.factor(as.numeric(y))
Xp <- X

library(CMA)
selg <- GeneSelection(y = yp, X = Xp, method = 't.test')@rankings[[1]][1:p]

# The 200 variables with the highest t-statistics are taken as the
# informative ones:
Xps <- Xp[, selg[1:p1]]

# The rest correspond to the non-informative ones:
Xpno <- Xp[, selg[(p1+1):p]]


# Subset the informative submatrices according to the two groups:
Xps1 <- Xps[yp==1,]
Xps2 <- Xps[yp==2,]


# Mean differences for the informative variables:
diffm <- colMeans(Xps2) - colMeans(Xps1)

# Estimate covariance matrices of the informative variables
# separated after 'group':
covs1 <- cov(Xps1)
covs2 <- cov(Xps2)

# 'mvtnorm'-package used for simulating from the multivariate
# normal distribution:
library(mvtnorm)

# 'corpcor'-package used for finding nearest positive definite
# matrices:
library(corpcor)

# Make the estimated covariance matrices positive definite:
covs1inf <- make.positive.definite(covs1)
covs2inf <- make.positive.definite(covs2)


# Covariance matrices for non-informative variables:
covsnoninf <- list()

for(bb in 0:((p-p1)/p1-1))
  covsnoninf[[bb+1]] <- make.positive.definite(cov(Xpno[,(bb*p1+1):((1+bb)*p1)]))


# Save the simulation parameters:

save(diffm, covsnoninf, covs1inf, covs2inf, p, p1, file='~/SimulationCVIIM/confs/simucorbigsignal.RData')





# Check the strength of the signal:

# Here we simulate a couple of data sets and perform some full CV repititions
# to check whether the error has the desired size:



# For Variable selection + Linear Discriminant Analysis:


# n = 50:


testerrorslda <- 0

for(i in 1:10) {

n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



nvar <- 10
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

testerrorslda[i] <- mean(errornvar10K3addon)

}



# n = 100:

testerrorslda2 <- 0

for(i in 1:10) {


n <- 100

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))




nvar <- 10
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

testerrorslda2[i] <- mean(errornvar10K3addon)

}


# Look at the errors:

boxplot(testerrorslda, testerrorslda2)

# --> OK.






# For Variable selection + Diagonal Linear Discriminant Analysis:


# n = 50:

testerrorsdlda <- 0

for(i in 1:10) {


n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))


nvar <- floor(ncol(X)/2)
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesellist = list(method = "t.test"), delta = 0)

errornvarbig2K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)


testerrorsdlda[i] <- mean(errornvarbig2K3addon)

}



# n = 100:

testerrorsdlda2 <- 0

for(i in 1:10) {


n <- 100

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



nvar <- floor(ncol(X)/2)
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesellist = list(method = "t.test"), delta = 0)

errornvarbig2K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)


testerrorsdlda2[i] <- mean(errornvarbig2K3addon)

}



# Look at the errors:

boxplot(testerrorsdlda, testerrorsdlda2)

# --> OK.





# Now we check whether also the estimated measure-values are in
# a meaningful range:


# Therefore we perform also incomplete CV in addition to full CV and 
# increase also the number of CV replications to get more stable 
# CVIIM estimates.


# For Variable selection + Linear Discriminant Analysis:

errorsfull <- errorsincompl <- matrix(nrow=5, ncol=5)

for(i in 1:5) {

n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



for(j in 1:5) {

nvar <- 10
K <- 3
niter <- 50


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsfull[i,j] <- mean(errornvar10K3addon)



varinds <- GeneSelection(X, y, method = "t.test")@rankings[[1]][1, 1:nvar]
Xsub <- X[, varinds]

nestedcrossval <- classification(Xsub, y, learningsets = cvdatasets, 
  classifier = ldaCMA)

errornvar10K3full <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsincompl[i,j] <- mean(errornvar10K3full)


}

}


# Look at the CVIIM-estimates - each boxplot belongs to one simulated
# data set:

boxplot(t(1 - errorsincompl/errorsfull))

# --> OK.

boxplot(t(errorsincompl))
boxplot(t(errorsfull))





# For Variable selection + Diagonal Linear Discriminant Analysis:

errorsdfull <- errorsdincompl <- matrix(nrow=5, ncol=5)


for(i in 1:5) {

n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



for(j in 1:5) {

nvar <- floor(ncol(X)/2)
K <- 3
niter <- 50


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesellist = list(method = "t.test"), delta = 0)

errornvarbig2K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsdfull[i,j] <- mean(errornvarbig2K3addon)



varinds <- GeneSelection(X, y, method = "t.test")@rankings[[1]][1, 1:nvar]
Xsub <- X[, varinds]

nestedcrossval <- classification(Xsub, y, learningsets = cvdatasets, 
  classifier = scdaCMA, delta=0)

errornvarbig2K3full <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsdincompl[i,j] <- mean(errornvarbig2K3full)

}

}


# Look at the CVIIM-estimates - each boxplot belongs to one simulated
# data set:

boxplot(t(1 - errorsdincompl/errorsdfull))

# --> OK.

boxplot(t(errorsdincompl))
boxplot(t(errorsdfull))













# Scenario with block-diagonal covariance matrix and weaker signal:
###################################################################

# set seed for reproducibility:
set.seed(32459)

# number of variables to simulate:
p <- 2000

# Load the data set:
load("./CVIIMCodeData/DataRetrieval/DataSetsPCA/ProstatecTranscr.Rda")

# Proportion of variables with signal:
signal <- 0.05

# Number of variables with signal:
p1 <- ceiling(p*signal)

# Number of variables without signal:
p2 <- p - p1


# We preselect the 2000 variables with the highest t-statistics
# from two-sample-tests between the two groups:

yp <- as.factor(as.numeric(y))
Xp <- X

library(CMA)
selg <- GeneSelection(y = yp, X = Xp, method = 't.test')@rankings[[1]][1:p]

# The 100 variables with the highest t-statistics are taken as the
# informative ones:
Xps <- Xp[, selg[1:p1]]

# The rest correspond to the non-informative ones:
Xpno <- Xp[, selg[(p1+1):p]]


# Subset the informative submatrices according to the two groups:
Xps1 <- Xps[yp==1,]
Xps2 <- Xps[yp==2,]


# Mean differences for the informative variables.
# The signal is weakened by multiplying 0.7 to the class
# differences:
diffm <- 0.7*(colMeans(Xps2) - colMeans(Xps1))


# Estimate covariance matrices of the informative variables
# separated after 'group':
covs1 <- cov(Xps1)
covs2 <- cov(Xps2)

# 'mvtnorm'-package used for simulating from the multivariate
# normal distribution:
library(mvtnorm)

# 'corpcor'-package used for finding nearest positive definite
# matrices:
library(corpcor)

# Make the estimated covariance matrices positive definite:
covs1inf <- make.positive.definite(covs1)
covs2inf <- make.positive.definite(covs2)


# Covariance matrices for non-informative variables:
covsnoninf <- list()

for(bb in 0:((p-p1)/p1-1))
  covsnoninf[[bb+1]] <- make.positive.definite(cov(Xpno[,(bb*p1+1):((1+bb)*p1)]))


# Save the simulation parameters:

save(diffm, covsnoninf, covs1inf, covs2inf, p, p1, file='~/SimulationCVIIM/confs/simucorweaksignal.RData')






# Here we do the same as for the first simulation scenario,
# wherefore this is not commented throughout:


testerrorsweaklda <- 0

for(i in 1:10) {


n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))


nvar <- 10
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

testerrorsweaklda[i] <- mean(errornvar10K3addon)

}




testerrorsweaklda2 <- 0

for(i in 1:10) {


n <- 100

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))




nvar <- 10
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

testerrorsweaklda2[i] <- mean(errornvar10K3addon)

}


# Look at the errors:

boxplot(testerrorsweaklda, testerrorsweaklda2)

# --> OK.







testerrorsweakdlda <- 0

for(i in 1:10) {


n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))


nvar <- floor(ncol(X)/2)
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesellist = list(method = "t.test"), delta = 0)

errornvarbig2K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)


testerrorsweakdlda[i] <- mean(errornvarbig2K3addon)

}




testerrorsweakdlda2 <- 0

for(i in 1:10) {


n <- 100

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))




nvar <- floor(ncol(X)/2)
K <- 3
niter <- 10


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesellist = list(method = "t.test"), delta = 0)

errornvarbig2K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)


testerrorsweakdlda2[i] <- mean(errornvarbig2K3addon)

}


# Look at the errors:

boxplot(testerrorsweakdlda, testerrorsweakdlda2)

# ---> OK.






errorsweakfull <- errorsweakincompl <- matrix(nrow=5, ncol=5)


for(i in 1:5) {

n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



for(j in 1:5) {

nvar <- 10
K <- 3
niter <- 50


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsweakfull[i,j] <- mean(errornvar10K3addon)



varinds <- GeneSelection(X, y, method = "t.test")@rankings[[1]][1, 1:nvar]
Xsub <- X[, varinds]

nestedcrossval <- classification(Xsub, y, learningsets = cvdatasets, 
  classifier = ldaCMA)

errornvar10K3full <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsweakincompl[i,j] <- mean(errornvar10K3full)


}

}


# Look at the CVIIM-estimates - each boxplot belongs to one simulated
# data set:

boxplot(t(1 - errorsweakincompl/errorsweakfull))

# --> OK.

boxplot(t(errorsweakincompl))
boxplot(t(errorsweakfull))






errorsweakfull <- errorsweakincompl <- matrix(nrow=5, ncol=5)


for(i in 1:5) {

n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



for(j in 1:5) {

nvar <- 10
K <- 3
niter <- 50


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = ldaCMA, nbgene = nvar, genesellist = list(method = "t.test"))

errornvar10K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsweakfull[i,j] <- mean(errornvar10K3addon)



varinds <- GeneSelection(X, y, method = "t.test")@rankings[[1]][1, 1:nvar]
Xsub <- X[, varinds]

nestedcrossval <- classification(Xsub, y, learningsets = cvdatasets, 
  classifier = ldaCMA)

errornvar10K3full <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsweakincompl[i,j] <- mean(errornvar10K3full)


}

}


# Look at the CVIIM-estimates - each boxplot belongs to one simulated
# data set:

boxplot(t(1 - errorsweakincompl/errorsweakfull))

# --> OK.

boxplot(t(errorsweakincompl))
boxplot(t(errorsweakfull))







errorsweakdfull <- errorsweakdincompl <- matrix(nrow=5, ncol=5)


for(i in 1:5) {

n <- 50

# Simulate informative variables for n/2 observations of class 1 and 2:
X1 <- rmvnorm(n = n/2, mean = rep(0, p1), sigma = covs1inf)
X2 <- rmvnorm(n = n/2, mean = diffm, sigma = covs2inf)
X <- rbind(X1, X2)


# Simulate non-informative variables for n observations of both classes:
for(bb in 0:((p-p1)/p1-1)){
  X <- cbind(X, rmvnorm(n = n, mean = rep(0, p1), sigma = covsnoninf[[bb+1]]))
}

# Generate response variable:
y <- factor(rep(c(1,2),each=n/2))



for(j in 1:5) {

nvar <- floor(ncol(X)/2)
K <- 3
niter <- 50


cvdatasets <- GenerateLearningsets(y = y, method = "CV", fold = K, niter=niter)


nestedcrossval <- classification(X, y, learningsets = cvdatasets, 
  classifier = scdaCMA, nbgene = nvar, genesellist = list(method = "t.test"), delta = 0)

errornvarbig2K3addon <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsweakdfull[i,j] <- mean(errornvarbig2K3addon)



varinds <- GeneSelection(X, y, method = "t.test")@rankings[[1]][1, 1:nvar]
Xsub <- X[, varinds]

nestedcrossval <- classification(Xsub, y, learningsets = cvdatasets, 
  classifier = scdaCMA, delta=0)

errornvarbig2K3full <- matrix(nrow=K, ncol=niter, data=evaluation(nestedcrossval, measure = "misclassification")@score)

errorsweakdincompl[i,j] <- mean(errornvarbig2K3full)

}

}


# Look at the CVIIM-estimates - each boxplot belongs to one simulated
# data set:

boxplot(t(1 - errorsweakdincompl/errorsweakdfull))

# --> OK.

boxplot(t(errorsweakdincompl))
boxplot(t(errorsweakdfull))
